@@ -10,18 +10,18 @@
  * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
  */
 
-#include "tcl.h"
 #include "tclPort.h"
 #include <sys/timeb.h>
 
 #undef timezone
 
 int
-gettimeofday(tp, tz)
-struct timeval *tp;
-struct timezone *tz;
+gettimeofday(
+    struct timeval *tp,
+    struct timezone *tz)
 {
     struct timeb t;
+
     ftime(&t);
     tp->tv_sec = t.time;
     tp->tv_usec = t. millitm * 1000;
