@@ -39,15 +39,6 @@
  * Things that regcustom.h might override.
  */
 
-/* standard header files (NULL is a reasonable indicator for them) */
-#ifndef NULL
-#include <stdio.h>
-#include <stdlib.h>
-#include <ctype.h>
-#include <limits.h>
-#include <string.h>
-#endif
-
 /* assertions */
 #ifndef assert
 #ifndef REG_DEBUG
@@ -75,11 +66,6 @@
 #define	NOPARMS	void		/* for empty parm lists */
 #endif
 
-/* const */
-#ifndef CONST
-#define	CONST	const		/* for old compilers, might be empty */
-#endif
-
 /* function-pointer declarator */
 #ifndef FUNCPTR
 #if __STDC__ >= 1
@@ -101,9 +87,6 @@
 #endif
 
 /* want size of a char in bits, and max value in bounded quantifiers */
-#ifndef CHAR_BIT
-#include <limits.h>
-#endif
 #ifndef _POSIX2_RE_DUP_MAX
 #define	_POSIX2_RE_DUP_MAX 255	/* normally from <limits.h> */
 #endif
@@ -116,7 +99,7 @@
 #define	xxx		1
 
 #define	DUPMAX	_POSIX2_RE_DUP_MAX
-#define	DUPINF	(DUPMAX+1)
+#define	INFINITY	(DUPMAX+1)
 
 #define	REMAGIC	0xfed7		/* magic number for main struct */
 
@@ -265,10 +248,8 @@ struct arc {
     struct state *from;		/* where it's from (and contained within) */
     struct state *to;		/* where it's to */
     struct arc *outchain;	/* *from's outs chain or free chain */
-    struct arc *outchainRev;	/* back-link in *from's outs chain */
-#define	freechain outchain	/* we do not maintain "freechainRev" */
+#define	freechain	outchain
     struct arc *inchain;	/* *to's ins chain */
-    struct arc *inchainRev;	/* back-link in *to's ins chain */
     struct arc *colorchain;	/* color's arc chain */
     struct arc *colorchainRev;	/* back-link in color's arc chain */
 };
@@ -386,7 +367,7 @@ struct subre {
  */
 
 struct fns {
-    VOID FUNCPTR(free, (regex_t *));
+    void FUNCPTR(free, (regex_t *));
 };
 
 /*
@@ -403,7 +384,7 @@ struct guts {
     struct cnfa search;		/* for fast preliminary search */
     int ntree;
     struct colormap cmap;
-    int FUNCPTR(compare, (CONST chr *, CONST chr *, size_t));
+    int FUNCPTR(compare, (const chr *, const chr *, size_t));
     struct subre *lacons;	/* lookahead-constraint vector */
     int nlacons;		/* size of lacons */
 };
