@@ -43,6 +43,11 @@ static char *		EnvTraceProc(ClientData clientData, Tcl_Interp *interp,
 static void		ReplaceString(const char *oldStr, char *newStr);
 MODULE_SCOPE void	TclSetEnv(const char *name, const char *value);
 MODULE_SCOPE void	TclUnsetEnv(const char *name);
+
+#if defined(__CYGWIN__)
+    static void TclCygwinPutenv(char *string);
+#   define putenv TclCygwinPutenv
+#endif
 
 /*
  *----------------------------------------------------------------------
@@ -153,7 +158,8 @@ TclSetEnv(
     const char *value)		/* New value for variable (UTF-8). */
 {
     Tcl_DString envString;
-    int index, length, nameLength;
+    unsigned nameLength, valueLength;
+    int index, length;
     char *p, *oldValue;
     const char *p2;
 
@@ -175,12 +181,11 @@ TclSetEnv(
 	 */
 
 	if ((env.ourEnviron != environ) || (length+2 > env.ourEnvironSize)) {
-	    char **newEnviron = (char **)
-		    ckalloc(((unsigned) length + 5) * sizeof(char *));
+	    char **newEnviron = ckalloc((length + 5) * sizeof(char *));
 
 	    memcpy(newEnviron, environ, length * sizeof(char *));
 	    if ((env.ourEnvironSize != 0) && (env.ourEnviron != NULL)) {
-		ckfree((char *) env.ourEnviron);
+		ckfree(env.ourEnviron);
 	    }
 	    environ = env.ourEnviron = newEnviron;
 	    env.ourEnvironSize = length + 5;
@@ -210,7 +215,7 @@ TclSetEnv(
 	Tcl_DStringFree(&envString);
 
 	oldValue = environ[index];
-	nameLength = length;
+	nameLength = (unsigned) length;
     }
 
     /*
@@ -219,18 +224,19 @@ TclSetEnv(
      * and set the environ array value.
      */
 
-    p = ckalloc((unsigned) nameLength + strlen(value) + 2);
-    strcpy(p, name);
+    valueLength = strlen(value);
+    p = ckalloc(nameLength + valueLength + 2);
+    memcpy(p, name, nameLength);
     p[nameLength] = '=';
-    strcpy(p+nameLength+1, value);
+    memcpy(p+nameLength+1, value, valueLength+1);
     p2 = Tcl_UtfToExternalDString(NULL, p, -1, &envString);
 
     /*
      * Copy the native string to heap memory.
      */
 
-    p = ckrealloc(p, strlen(p2) + 1);
-    strcpy(p, p2);
+    p = ckrealloc(p, Tcl_DStringLength(&envString) + 1);
+    memcpy(p, p2, (unsigned) Tcl_DStringLength(&envString) + 1);
     Tcl_DStringFree(&envString);
 
 #ifdef USE_PUTENV
@@ -389,20 +395,21 @@ TclUnsetEnv(
      * that no = should be included, and Windows requires it.
      */
 
-#if defined(__WIN32__)
-    string = ckalloc((unsigned) length+2);
+#if defined(__WIN32__) || defined(__CYGWIN__)
+    string = ckalloc(length + 2);
     memcpy(string, name, (size_t) length);
     string[length] = '=';
     string[length+1] = '\0';
 #else
-    string = ckalloc((unsigned) length+1);
+    string = ckalloc(length + 1);
     memcpy(string, name, (size_t) length);
     string[length] = '\0';
 #endif /* WIN32 */
 
     Tcl_UtfToExternalDString(NULL, string, -1, &envString);
-    string = ckrealloc(string, (unsigned) Tcl_DStringLength(&envString)+1);
-    strcpy(string, Tcl_DStringValue(&envString));
+    string = ckrealloc(string, Tcl_DStringLength(&envString) + 1);
+    memcpy(string, Tcl_DStringValue(&envString),
+	    (unsigned) Tcl_DStringLength(&envString)+1);
     Tcl_DStringFree(&envString);
 
     putenv(string);
@@ -558,7 +565,7 @@ EnvTraceProc(
 	const char *value = TclGetEnv(name2, &valueString);
 
 	if (value == NULL) {
-	    return "no such variable";
+	    return (char *) "no such variable";
 	}
 	Tcl_SetVar2(interp, name1, name2, value, 0);
 	Tcl_DStringFree(&valueString);
@@ -635,11 +642,11 @@ ReplaceString(
 
 	const int growth = 5;
 
-	env.cache = (char **) ckrealloc((char *) env.cache,
+	env.cache = ckrealloc(env.cache,
 		(env.cacheSize + growth) * sizeof(char *));
 	env.cache[env.cacheSize] = newStr;
-	(void) memset(env.cache+env.cacheSize+1, (int) 0,
-		(size_t) (growth-1) * sizeof(char*));
+	(void) memset(env.cache+env.cacheSize+1, 0,
+		(size_t) (growth-1) * sizeof(char *));
 	env.cacheSize += growth;
     }
 }
@@ -674,7 +681,7 @@ TclFinalizeEnvironment(void)
      */
 
     if (env.cache) {
-	ckfree((char *) env.cache);
+	ckfree(env.cache);
 	env.cache = NULL;
 	env.cacheSize = 0;
 #ifndef USE_PUTENV
@@ -683,6 +690,98 @@ TclFinalizeEnvironment(void)
     }
 }
 
+#if defined(__CYGWIN__)
+
+/*
+ * When using cygwin, when an environment variable changes, we need to synch
+ * with both the cygwin environment (in case the application C code calls
+ * fork) and the Windows environment (in case the application TCL code calls
+ * exec, which calls the Windows CreateProcess function).
+ */
+DLLIMPORT extern void __stdcall SetEnvironmentVariableA(const char*, const char *);
+
+static void
+TclCygwinPutenv(
+    char *str)
+{
+    char *name, *value;
+
+    /*
+     * Get the name and value, so that we can change the environment variable
+     * for Windows.
+     */
+
+    name = alloca(strlen(str) + 1);
+    strcpy(name, str);
+    for (value=name ; *value!='=' && *value!='\0' ; ++value) {
+	/* Empty body */
+    }
+    if (*value == '\0') {
+	/* Can't happen. */
+	return;
+    }
+    *(value++) = '\0';
+    if (*value == '\0') {
+	value = NULL;
+    }
+
+    /*
+     * Set the cygwin environment variable.
+     */
+
+#undef putenv
+    if (value == NULL) {
+	unsetenv(name);
+    } else {
+	putenv(str);
+    }
+
+    /*
+     * Before changing the environment variable in Windows, if this is PATH,
+     * we need to convert the value back to a Windows style path.
+     *
+     * FIXME: The calling program may know it is running under windows, and
+     * may have set the path to a Windows path, or, worse, appended or
+     * prepended a Windows path to PATH.
+     */
+
+    if (strcmp(name, "PATH") != 0) {
+	/*
+	 * If this is Path, eliminate any PATH variable, to prevent any
+	 * confusion.
+	 */
+
+	if (strcmp(name, "Path") == 0) {
+	    SetEnvironmentVariableA("PATH", NULL);
+	    unsetenv("PATH");
+	}
+
+	SetEnvironmentVariableA(name, value);
+    } else {
+	char *buf;
+
+	/*
+	 * Eliminate any Path variable, to prevent any confusion.
+	 */
+
+	SetEnvironmentVariableA("Path", NULL);
+	unsetenv("Path");
+
+	if (value == NULL) {
+	    buf = NULL;
+	} else {
+	    int size;
+
+	    size = cygwin_conv_path_list(0, value, NULL, 0);
+	    buf = alloca(size + 1);
+	    cygwin_conv_path_list(0, value, buf, size);
+	}
+
+	SetEnvironmentVariableA(name, buf);
+    }
+}
+#endif /* __CYGWIN__ */
+
 /*
  * Local Variables:
  * mode: c
