@@ -116,7 +116,7 @@ static inline void	ResultAdd(ResultBuffer *r, unsigned char *buf,
  * transformations.
  */
 
-static Tcl_ChannelType transformChannelType = {
+static const Tcl_ChannelType transformChannelType = {
     "transform",		/* Type name. */
     TCL_CHANNEL_VERSION_5,	/* v5 channel */
     TransformCloseProc,		/* Close proc. */
@@ -210,27 +210,7 @@ struct TransformChannelData {
 				 * a transformation of incoming data. Also
 				 * serves as buffer of all data not yet
 				 * consumed by the reader. */
-    int refCount;
 };
-
-static void
-PreserveData(
-    TransformChannelData *dataPtr)
-{
-    dataPtr->refCount++;
-}
-
-static void
-ReleaseData(
-    TransformChannelData *dataPtr)
-{
-    if (--dataPtr->refCount) {
-	return;
-    }
-    ResultClear(&dataPtr->result);
-    Tcl_DecrRefCount(dataPtr->command);
-    ckfree((char *) dataPtr);
-}
 
 /*
  *----------------------------------------------------------------------
@@ -260,7 +240,6 @@ TclChannelTransform(
     Channel *chanPtr;		/* The actual channel. */
     ChannelState *statePtr;	/* State info for channel. */
     int mode;			/* Read/write mode of the channel. */
-    int objc;
     TransformChannelData *dataPtr;
     Tcl_DString ds;
 
@@ -268,12 +247,6 @@ TclChannelTransform(
 	return TCL_ERROR;
     }
 
-    if (TCL_OK != Tcl_ListObjLength(interp, cmdObjPtr, &objc)) {
-	Tcl_SetObjResult(interp,
-		Tcl_NewStringObj("-command value is not a list", -1));
-	return TCL_ERROR;
-    }
-
     chanPtr = (Channel *) chan;
     statePtr = chanPtr->state;
     chanPtr = statePtr->topChanPtr;
@@ -286,9 +259,8 @@ TclChannelTransform(
      * regime of the underlying channel and to use the same for us too.
      */
 
-    dataPtr = (TransformChannelData *) ckalloc(sizeof(TransformChannelData));
+    dataPtr = ckalloc(sizeof(TransformChannelData));
 
-    dataPtr->refCount = 1;
     Tcl_DStringInit(&ds);
     Tcl_GetChannelOption(interp, chan, "-blocking", &ds);
     dataPtr->readIsFlushed = 0;
@@ -298,6 +270,7 @@ TclChannelTransform(
     }
     Tcl_DStringFree(&ds);
 
+    dataPtr->self = chan;
     dataPtr->watchMask = 0;
     dataPtr->mode = mode;
     dataPtr->timer = NULL;
@@ -311,22 +284,21 @@ TclChannelTransform(
     dataPtr->self = Tcl_StackChannel(interp, &transformChannelType, dataPtr,
 	    mode, chan);
     if (dataPtr->self == NULL) {
-	Tcl_AppendResult(interp, "\nfailed to stack channel \"",
-		Tcl_GetChannelName(chan), "\"", NULL);
-	ReleaseData(dataPtr);
+	Tcl_AppendPrintfToObj(Tcl_GetObjResult(interp),
+		"\nfailed to stack channel \"%s\"", Tcl_GetChannelName(chan));
+	Tcl_DecrRefCount(dataPtr->command);
+	ResultClear(&dataPtr->result);
+	ckfree(dataPtr);
 	return TCL_ERROR;
     }
-    Tcl_Preserve(dataPtr->self);
 
     /*
      * At last initialize the transformation at the script level.
      */
 
-    PreserveData(dataPtr);
     if ((dataPtr->mode & TCL_WRITABLE) && ExecuteCallback(dataPtr, NULL,
 	    A_CREATE_WRITE, NULL, 0, TRANSMIT_DONT, P_NO_PRESERVE) != TCL_OK){
 	Tcl_UnstackChannel(interp, chan);
-	ReleaseData(dataPtr);
 	return TCL_ERROR;
     }
 
@@ -335,11 +307,9 @@ TclChannelTransform(
 	ExecuteCallback(dataPtr, NULL, A_DELETE_WRITE, NULL, 0, TRANSMIT_DONT,
 		P_NO_PRESERVE);
 	Tcl_UnstackChannel(interp, chan);
-	ReleaseData(dataPtr);
 	return TCL_ERROR;
     }
 
-    ReleaseData(dataPtr);
     return TCL_OK;
 }
 
@@ -380,10 +350,7 @@ ExecuteCallback(
     unsigned char *resBuf;
     Tcl_InterpState state = NULL;
     int res = TCL_OK;
-    Tcl_Obj *command = TclListObjCopy(NULL, dataPtr->command);
-    Tcl_Interp *eval = dataPtr->interp;
-
-    Tcl_Preserve(eval);
+    Tcl_Obj *command = Tcl_DuplicateObj(dataPtr->command);
 
     /*
      * Step 1, create the complete command to execute. Do this by appending
@@ -394,18 +361,26 @@ ExecuteCallback(
      */
 
     if (preserve == P_PRESERVE) {
-	state = Tcl_SaveInterpState(eval, res);
+	state = Tcl_SaveInterpState(dataPtr->interp, res);
     }
 
     Tcl_IncrRefCount(command);
-    Tcl_ListObjAppendElement(NULL, command, Tcl_NewStringObj((char *) op, -1));
+    res = Tcl_ListObjAppendElement(dataPtr->interp, command,
+	    Tcl_NewStringObj((char *) op, -1));
+    if (res != TCL_OK) {
+	goto cleanup;
+    }
 
     /*
      * Use a byte-array to prevent the misinterpretation of binary data coming
      * through as UTF while at the tcl level.
      */
 
-    Tcl_ListObjAppendElement(NULL, command, Tcl_NewByteArrayObj(buf, bufLen));
+    res = Tcl_ListObjAppendElement(dataPtr->interp, command,
+	    Tcl_NewByteArrayObj(buf, bufLen));
+    if (res != TCL_OK) {
+	goto cleanup;
+    }
 
     /*
      * Step 2, execute the command at the global level of the interpreter used
@@ -415,14 +390,13 @@ ExecuteCallback(
      * current interpreter. Don't copy if in preservation mode.
      */
 
-    res = Tcl_EvalObjEx(eval, command, TCL_EVAL_GLOBAL);
+    res = Tcl_EvalObjEx(dataPtr->interp, command, TCL_EVAL_GLOBAL);
     Tcl_DecrRefCount(command);
     command = NULL;
 
-    if ((res != TCL_OK) && (interp != NULL) && (eval != interp)
+    if ((res != TCL_OK) && (interp != NULL) && (dataPtr->interp != interp)
 	    && (preserve == P_NO_PRESERVE)) {
-	Tcl_SetObjResult(interp, Tcl_GetObjResult(eval));
-	Tcl_Release(eval);
+	Tcl_SetObjResult(interp, Tcl_GetObjResult(dataPtr->interp));
 	return res;
     }
 
@@ -437,26 +411,20 @@ ExecuteCallback(
 	break;
 
     case TRANSMIT_DOWN:
-	if (dataPtr->self == NULL) {
-	    break;
-	}
-	resObj = Tcl_GetObjResult(eval);
+	resObj = Tcl_GetObjResult(dataPtr->interp);
 	resBuf = Tcl_GetByteArrayFromObj(resObj, &resLen);
 	Tcl_WriteRaw(Tcl_GetStackedChannel(dataPtr->self), (char *) resBuf,
 		resLen);
 	break;
 
     case TRANSMIT_SELF:
-	if (dataPtr->self == NULL) {
-	    break;
-	}
-	resObj = Tcl_GetObjResult(eval);
+	resObj = Tcl_GetObjResult(dataPtr->interp);
 	resBuf = Tcl_GetByteArrayFromObj(resObj, &resLen);
 	Tcl_WriteRaw(dataPtr->self, (char *) resBuf, resLen);
 	break;
 
     case TRANSMIT_IBUF:
-	resObj = Tcl_GetObjResult(eval);
+	resObj = Tcl_GetObjResult(dataPtr->interp);
 	resBuf = Tcl_GetByteArrayFromObj(resObj, &resLen);
 	ResultAdd(&dataPtr->result, resBuf, resLen);
 	break;
@@ -466,16 +434,24 @@ ExecuteCallback(
 	 * Interpret result as integer number.
 	 */
 
-	resObj = Tcl_GetObjResult(eval);
-	TclGetIntFromObj(eval, resObj, &dataPtr->maxRead);
+	resObj = Tcl_GetObjResult(dataPtr->interp);
+	TclGetIntFromObj(dataPtr->interp, resObj, &dataPtr->maxRead);
 	break;
     }
 
-    Tcl_ResetResult(eval);
+    Tcl_ResetResult(dataPtr->interp);
     if (preserve == P_PRESERVE) {
-	(void) Tcl_RestoreInterpState(eval, state);
+	(void) Tcl_RestoreInterpState(dataPtr->interp, state);
+    }
+    return res;
+
+  cleanup:
+    if (preserve == P_PRESERVE) {
+	(void) Tcl_RestoreInterpState(dataPtr->interp, state);
+    }
+    if (command != NULL) {
+	Tcl_DecrRefCount(command);
     }
-    Tcl_Release(eval);
     return res;
 }
 
@@ -559,7 +535,6 @@ TransformCloseProc(
      * system rely on (f.e. signaling the close to interested parties).
      */
 
-    PreserveData(dataPtr);
     if (dataPtr->mode & TCL_WRITABLE) {
 	ExecuteCallback(dataPtr, interp, A_FLUSH_WRITE, NULL, 0,
 		TRANSMIT_DOWN, P_PRESERVE);
@@ -579,15 +554,14 @@ TransformCloseProc(
 	ExecuteCallback(dataPtr, interp, A_DELETE_READ, NULL, 0,
 		TRANSMIT_DONT, P_PRESERVE);
     }
-    ReleaseData(dataPtr);
 
     /*
      * General cleanup.
      */
 
-    Tcl_Release(dataPtr->self);
-    dataPtr->self = NULL;
-    ReleaseData(dataPtr);
+    ResultClear(&dataPtr->result);
+    Tcl_DecrRefCount(dataPtr->command);
+    ckfree(dataPtr);
     return TCL_OK;
 }
 
@@ -622,7 +596,7 @@ TransformInputProc(
      * Should assert(dataPtr->mode & TCL_READABLE);
      */
 
-    if (toRead == 0 || dataPtr->self == NULL) {
+    if (toRead == 0) {
 	/*
 	 * Catch a no-op.
 	 */
@@ -632,7 +606,6 @@ TransformInputProc(
     gotBytes = 0;
     downChan = Tcl_GetStackedChannel(dataPtr->self);
 
-    PreserveData(dataPtr);
     while (toRead > 0) {
 	/*
 	 * Loop until the request is satisfied (or no data is available from
@@ -650,7 +623,7 @@ TransformInputProc(
 	     * break out of the loop and return to the caller.
 	     */
 
-	    break;
+	    return gotBytes;
 	}
 
 	/*
@@ -674,7 +647,7 @@ TransformInputProc(
 	    }
 	} /* else: 'maxRead < 0' == Accept the current value of toRead. */
 	if (toRead <= 0) {
-	    break;
+	    return gotBytes;
 	}
 
 	/*
@@ -683,41 +656,43 @@ TransformInputProc(
 
 	read = Tcl_ReadRaw(downChan, buf, toRead);
 	if (read < 0) {
+	    /*
+	     * Report errors to caller. EAGAIN is a special situation. If we
+	     * had some data before we report that instead of the request to
+	     * re-try.
+	     */
 
-	    if (Tcl_InputBlocked(downChan) && (gotBytes > 0)) {
-		/*
-		 * Zero bytes available from downChan because blocked.
-		 * But nonzero bytes already copied, so total is a
-		 * valid blocked short read. Return to caller.
-		 */
-
-		break;
+	    if ((Tcl_GetErrno() == EAGAIN) && (gotBytes > 0)) {
+		return gotBytes;
 	    }
 
-	    /*
-	     * Either downChan is not blocked (there's a real error).
-	     * or it is and there are no bytes copied yet.  In either
-	     * case we want to pass the "error" along to the caller,
-	     * either to report an error, or to signal to the caller
-	     * that zero bytes are available because blocked.
-	     */
-
 	    *errorCodePtr = Tcl_GetErrno();
-	    gotBytes = -1;
-	    break;
+	    return -1;
 	} else if (read == 0) {
-
 	    /*
-	     * Zero returned from Tcl_ReadRaw() always indicates EOF
-	     * on the down channel.
+	     * Check wether we hit on EOF in the underlying channel or not. If
+	     * not differentiate between blocking and non-blocking modes. In
+	     * non-blocking mode we ran temporarily out of data. Signal this
+	     * to the caller via EWOULDBLOCK and error return (-1). In the
+	     * other cases we simply return what we got and let the caller
+	     * wait for more. On the other hand, if we got an EOF we have to
+	     * convert and flush all waiting partial data.
 	     */
 
+	    if (!Tcl_Eof(downChan)) {
+		if ((gotBytes == 0) && (dataPtr->flags & CHANNEL_ASYNC)) {
+		    *errorCodePtr = EWOULDBLOCK;
+		    return -1;
+		}
+		return gotBytes;
+	    }
+
 	    if (dataPtr->readIsFlushed) {
 		/*
 		 * Already flushed, nothing to do anymore.
 		 */
 
-		break;
+		return gotBytes;
 	    }
 
 	    dataPtr->readIsFlushed = 1;
@@ -729,7 +704,7 @@ TransformInputProc(
 		 * We had nothing to flush.
 		 */
 
-		break;
+		return gotBytes;
 	    }
 
 	    continue;		/* at: while (toRead > 0) */
@@ -743,11 +718,9 @@ TransformInputProc(
 	if (ExecuteCallback(dataPtr, NULL, A_READ, UCHARP(buf), read,
 		TRANSMIT_IBUF, P_PRESERVE) != TCL_OK) {
 	    *errorCodePtr = EINVAL;
-	    gotBytes = -1;
-	    break;
+	    return -1;
 	}
     } /* while toRead > 0 */
-    ReleaseData(dataPtr);
 
     return gotBytes;
 }
@@ -789,13 +762,11 @@ TransformOutputProc(
 	return 0;
     }
 
-    PreserveData(dataPtr);
     if (ExecuteCallback(dataPtr, NULL, A_WRITE, UCHARP(buf), toWrite,
 	    TRANSMIT_DOWN, P_NO_PRESERVE) != TCL_OK) {
 	*errorCodePtr = EINVAL;
-	toWrite = -1;
+	return -1;
     }
-    ReleaseData(dataPtr);
 
     return toWrite;
 }
@@ -829,7 +800,7 @@ TransformSeekProc(
 {
     TransformChannelData *dataPtr = instanceData;
     Tcl_Channel parent = Tcl_GetStackedChannel(dataPtr->self);
-    Tcl_ChannelType *parentType = Tcl_GetChannelType(parent);
+    const Tcl_ChannelType *parentType = Tcl_GetChannelType(parent);
     Tcl_DriverSeekProc *parentSeekProc = Tcl_ChannelSeekProc(parentType);
 
     if ((offset == 0) && (mode == SEEK_CUR)) {
@@ -848,7 +819,6 @@ TransformSeekProc(
      * request down, unchanged.
      */
 
-    PreserveData(dataPtr);
     if (dataPtr->mode & TCL_WRITABLE) {
 	ExecuteCallback(dataPtr, NULL, A_FLUSH_WRITE, NULL, 0, TRANSMIT_DOWN,
 		P_NO_PRESERVE);
@@ -860,7 +830,6 @@ TransformSeekProc(
 	ResultClear(&dataPtr->result);
 	dataPtr->readIsFlushed = 0;
     }
-    ReleaseData(dataPtr);
 
     return parentSeekProc(Tcl_GetChannelInstanceData(parent), offset, mode,
 	    errorCodePtr);
@@ -895,7 +864,7 @@ TransformWideSeekProc(
 {
     TransformChannelData *dataPtr = instanceData;
     Tcl_Channel parent = Tcl_GetStackedChannel(dataPtr->self);
-    Tcl_ChannelType *parentType	= Tcl_GetChannelType(parent);
+    const Tcl_ChannelType *parentType	= Tcl_GetChannelType(parent);
     Tcl_DriverSeekProc *parentSeekProc = Tcl_ChannelSeekProc(parentType);
     Tcl_DriverWideSeekProc *parentWideSeekProc =
 	    Tcl_ChannelWideSeekProc(parentType);
@@ -921,7 +890,6 @@ TransformWideSeekProc(
      * request down, unchanged.
      */
 
-    PreserveData(dataPtr);
     if (dataPtr->mode & TCL_WRITABLE) {
 	ExecuteCallback(dataPtr, NULL, A_FLUSH_WRITE, NULL, 0, TRANSMIT_DOWN,
 		P_NO_PRESERVE);
@@ -933,7 +901,6 @@ TransformWideSeekProc(
 	ResultClear(&dataPtr->result);
 	dataPtr->readIsFlushed = 0;
     }
-    ReleaseData(dataPtr);
 
     /*
      * If we have a wide seek capability, we should stick with that.
@@ -1088,9 +1055,6 @@ TransformWatchProc(
      * unchanged.
      */
 
-    if (dataPtr->self == NULL) {
-	return;
-    }
     downChan = Tcl_GetStackedChannel(dataPtr->self);
 
     Tcl_GetChannelType(downChan)->watchProc(
@@ -1263,7 +1227,7 @@ ResultClear(
     r->used = 0;
 
     if (r->allocated) {
-	ckfree((char *) r->buf);
+	ckfree(r->buf);
 	r->buf = NULL;
 	r->allocated = 0;
     }
@@ -1407,10 +1371,10 @@ ResultAdd(
 
 	if (r->allocated == 0) {
 	    r->allocated = toWrite + INCREMENT;
-	    r->buf = UCHARP(ckalloc(r->allocated));
+	    r->buf = ckalloc(r->allocated);
 	} else {
 	    r->allocated += toWrite + INCREMENT;
-	    r->buf = UCHARP(ckrealloc((char *) r->buf, r->allocated));
+	    r->buf = ckrealloc(r->buf, r->allocated);
 	}
     }
 
