@@ -1,5 +1,3 @@
-#define AT_FORK_INIT_VALUE 0
-#define RESET_ATFORK_MUTEX 1
 /*
  * tclUnixNotify.c --
  *
@@ -18,14 +16,6 @@
 				 * in tclMacOSXNotify.c */
 #include <signal.h>
 
-/*
- * This code does deep stub magic to allow replacement of the notifier at
- * runtime.
- */
-
-extern TclStubs tclStubs;
-extern Tcl_NotifierProcs tclOriginalNotifier;
-
 /*
  * This structure is used to keep track of the notifier info for a registered
  * file.
@@ -107,11 +97,10 @@ typedef struct ThreadSpecificData {
 	 * by sending this event. */
     void *hwnd;			/* Messaging window. */
 #else /* !__CYGWIN__ */
-    pthread_cond_t waitCV;	/* Any other thread alerts a notifier that an
+    Tcl_Condition waitCV;	/* Any other thread alerts a notifier that an
 				 * event is ready to be processed by signaling
 				 * this condition variable. */
 #endif /* __CYGWIN__ */
-    int waitCVinitialized;	/* Variable to flag initialization of the structure */
     int eventReady;		/* True if an event is ready to be processed.
 				 * Used as condition flag together with waitCV
 				 * above. */
@@ -130,6 +119,15 @@ static Tcl_ThreadDataKey dataKey;
 
 static int notifierCount = 0;
 
+/*
+ * The following static stores the process ID of the initialized notifier
+ * thread. If it changes, we have passed a fork and we should start a new
+ * notifier thread.
+ *
+ * You must hold the notifierMutex lock before accessing this variable.
+ */
+static pid_t processIDInitialized = 0;
+
 /*
  * The following variable points to the head of a doubly-linked list of
  * ThreadSpecificData structures for all threads that are currently waiting on
@@ -160,15 +158,7 @@ static int triggerPipe = -1;
  * The notifierMutex locks access to all of the global notifier state.
  */
 
-pthread_mutex_t notifierInitMutex = PTHREAD_MUTEX_INITIALIZER;
-pthread_mutex_t notifierMutex     = PTHREAD_MUTEX_INITIALIZER;
-/*
- * The following static indicates if the notifier thread is running.
- *
- * You must hold the notifierInitMutex before accessing this variable.
- */
-
-static int notifierThreadRunning = 0;
+TCL_DECLARE_MUTEX(notifierMutex)
 
 /*
  * The notifier thread signals the notifierCV when it has finished
@@ -176,7 +166,7 @@ static int notifierThreadRunning = 0;
  * terminates.
  */
 
-static pthread_cond_t notifierCV = PTHREAD_COND_INITIALIZER;
+static Tcl_Condition notifierCV;
 
 /*
  * The pollState bits
@@ -204,15 +194,15 @@ static Tcl_ThreadId notifierThread;
 
 #ifdef TCL_THREADS
 static void	NotifierThreadProc(ClientData clientData);
-#if defined(HAVE_PTHREAD_ATFORK)
-static int	atForkInit = AT_FORK_INIT_VALUE;
+#ifdef HAVE_PTHREAD_ATFORK
+static int	atForkInit = 0;
 static void	AtForkPrepare(void);
 static void	AtForkParent(void);
 static void	AtForkChild(void);
 #endif /* HAVE_PTHREAD_ATFORK */
 #endif /* TCL_THREADS */
 static int	FileHandlerEventProc(Tcl_Event *evPtr, int flags);
-
+
 /*
  * Import of Windows API when building threaded with Cygwin.
  */
@@ -268,50 +258,6 @@ static DWORD __stdcall	NotifierProc(void *hwnd, unsigned int message,
 			    void *wParam, void *lParam);
 #endif /* TCL_THREADS && __CYGWIN__ */
 
-#if TCL_THREADS
-/*
- *----------------------------------------------------------------------
- *
- * StartNotifierThread --
- *
- *	Start a notfier thread and wait for the notifier pipe to be created.
- *
- * Results:
- *	None.
- *
- * Side effects:
- *	Running Thread.
- *
- *----------------------------------------------------------------------
- */
-static void
-StartNotifierThread(const char *proc)
-{
-    if (!notifierThreadRunning) {
-	pthread_mutex_lock(&notifierInitMutex);
-	if (!notifierThreadRunning) {
-	    if (TclpThreadCreate(&notifierThread, NotifierThreadProc, NULL,
-		    TCL_THREAD_STACK_DEFAULT, TCL_THREAD_JOINABLE) != TCL_OK) {
-		Tcl_Panic("%s: unable to start notifier thread", proc);
-	    }
-
-	    pthread_mutex_lock(&notifierMutex);
-	    /*
-	     * Wait for the notifier pipe to be created.
-	     */
-
-	    while (triggerPipe < 0) {
-		pthread_cond_wait(&notifierCV, &notifierMutex);
-	    }
-	    pthread_mutex_unlock(&notifierMutex);
-
-	    notifierThreadRunning = 1;
-	}
-	pthread_mutex_unlock(&notifierInitMutex);
-    }
-}
-#endif /* TCL_THREADS */
-
 /*
  *----------------------------------------------------------------------
  *
@@ -331,64 +277,66 @@ StartNotifierThread(const char *proc)
 ClientData
 Tcl_InitNotifier(void)
 {
-    ThreadSpecificData *tsdPtr = TCL_TSD_INIT(&dataKey);
+    if (tclNotifierHooks.initNotifierProc) {
+	return tclNotifierHooks.initNotifierProc();
+    } else {
+	ThreadSpecificData *tsdPtr = TCL_TSD_INIT(&dataKey);
 
 #ifdef TCL_THREADS
-    tsdPtr->eventReady = 0;
+	tsdPtr->eventReady = 0;
 
-    /*
-     * Initialize thread specific condition variable for this thread.
-     */
-    if (tsdPtr->waitCVinitialized == 0) {
-#ifdef __CYGWIN__
-	WNDCLASS class;
+	/*
+	 * Start the Notifier thread if necessary.
+	 */
 
-	class.style = 0;
-	class.cbClsExtra = 0;
-	class.cbWndExtra = 0;
-	class.hInstance = TclWinGetTclInstance();
-	class.hbrBackground = NULL;
-	class.lpszMenuName = NULL;
-	class.lpszClassName = L"TclNotifier";
-	class.lpfnWndProc = NotifierProc;
-	class.hIcon = NULL;
-	class.hCursor = NULL;
+	Tcl_MutexLock(&notifierMutex);
+#ifdef HAVE_PTHREAD_ATFORK
+	/*
+	 * Install pthread_atfork handlers to reinitialize the notifier in the
+	 * child of a fork.
+	 */
 
-	RegisterClassW(&class);
-	tsdPtr->hwnd = CreateWindowExW(NULL, class.lpszClassName,
-		class.lpszClassName, 0, 0, 0, 0, 0, NULL, NULL,
-		TclWinGetTclInstance(), NULL);
-	tsdPtr->event = CreateEventW(NULL, 1 /* manual */,
-		0 /* !signaled */, NULL);
-#else
-	pthread_cond_init(&tsdPtr->waitCV, NULL);
-#endif /* __CYGWIN__ */
-	tsdPtr->waitCVinitialized = 1;
-    }
+	if (!atForkInit) {
+	    int result = pthread_atfork(AtForkPrepare, AtForkParent, AtForkChild);
 
-    pthread_mutex_lock(&notifierInitMutex);
-#if defined(HAVE_PTHREAD_ATFORK)
-    /*
-     * Install pthread_atfork handlers to clean up the notifier in the
-     * child of a fork.
-     */
-
-    if (!atForkInit) {
-	int result = pthread_atfork(AtForkPrepare, AtForkParent, AtForkChild);
-
-	if (result) {
-	    Tcl_Panic("Tcl_InitNotifier: pthread_atfork failed");
+	    if (result) {
+		Tcl_Panic("Tcl_InitNotifier: pthread_atfork failed");
+	    }
+	    atForkInit = 1;
+	}
+#endif
+	/*
+	 * Check if my process id changed, e.g. I was forked
+	 * In this case, restart the notifier thread and close the
+	 * pipe to the original notifier thread
+	 */
+	if (notifierCount > 0 && processIDInitialized != getpid()) {
+	    notifierCount = 0;
+	    processIDInitialized = 0;
+	    close(triggerPipe);
+	    triggerPipe = -1;
+	}
+	if (notifierCount == 0) {
+	    if (TclpThreadCreate(&notifierThread, NotifierThreadProc, NULL,
+		    TCL_THREAD_STACK_DEFAULT, TCL_THREAD_JOINABLE) != TCL_OK) {
+		Tcl_Panic("Tcl_InitNotifier: unable to start notifier thread");
+	    }
+	    processIDInitialized = getpid();
 	}
-	atForkInit = 1;
-    }
-#endif /* HAVE_PTHREAD_ATFORK */
-
 	notifierCount++;
 
-	pthread_mutex_unlock(&notifierInitMutex);
+	/*
+	 * Wait for the notifier pipe to be created.
+	 */
 
+	while (triggerPipe < 0) {
+	    Tcl_ConditionWait(&notifierCV, &notifierMutex, NULL);
+	}
+
+	Tcl_MutexUnlock(&notifierMutex);
 #endif /* TCL_THREADS */
-    return (ClientData) tsdPtr;
+	return tsdPtr;
+    }
 }
 
 /*
@@ -413,54 +361,69 @@ void
 Tcl_FinalizeNotifier(
     ClientData clientData)		/* Not used. */
 {
+    if (tclNotifierHooks.finalizeNotifierProc) {
+	tclNotifierHooks.finalizeNotifierProc(clientData);
+	return;
+    } else {
 #ifdef TCL_THREADS
-    ThreadSpecificData *tsdPtr = TCL_TSD_INIT(&dataKey);
+	ThreadSpecificData *tsdPtr = TCL_TSD_INIT(&dataKey);
 
-    pthread_mutex_lock(&notifierInitMutex);
-    notifierCount--;
+	Tcl_MutexLock(&notifierMutex);
+	notifierCount--;
 
-    /*
-     * If this is the last thread to use the notifier, close the notifier
-     * pipe and wait for the background thread to terminate.
-     */
+	/*
+	 * If this is the last thread to use the notifier, close the notifier
+	 * pipe and wait for the background thread to terminate.
+	 */
 
-    if (notifierCount == 0) {
+	if (notifierCount == 0) {
+	    int result;
+
+	    if (triggerPipe < 0) {
+		Tcl_Panic("Tcl_FinalizeNotifier: %s",
+			"notifier pipe not initialized");
+	    }
+
+	    /*
+	     * Send "q" message to the notifier thread so that it will
+	     * terminate. The notifier will return from its call to select()
+	     * and notice that a "q" message has arrived, it will then close
+	     * its side of the pipe and terminate its thread. Note the we can
+	     * not just close the pipe and check for EOF in the notifier thread
+	     * because if a background child process was created with exec,
+	     * select() would not register the EOF on the pipe until the child
+	     * processes had terminated. [Bug: 4139] [Bug: 1222872]
+	     */
 
-	if (triggerPipe != -1) {
 	    if (write(triggerPipe, "q", 1) != 1) {
 		Tcl_Panic("Tcl_FinalizeNotifier: %s",
 			"unable to write q to triggerPipe");
 	    }
 	    close(triggerPipe);
-	    while(triggerPipe != -1) {
-		pthread_cond_wait(&notifierCV, &notifierMutex);
+	    while(triggerPipe >= 0) {
+		Tcl_ConditionWait(&notifierCV, &notifierMutex, NULL);
 	    }
-	    if (notifierThreadRunning) {
-		int result = pthread_join((pthread_t) notifierThread, NULL);
 
-		if (result) {
-		    Tcl_Panic("Tcl_FinalizeNotifier: unable to join notifier "
-			    "thread");
-		}
-		notifierThreadRunning = 0;
+	    result = Tcl_JoinThread(notifierThread, NULL);
+	    if (result) {
+		Tcl_Panic("Tcl_FinalizeNotifier: %s",
+			"unable to join notifier thread");
 	    }
 	}
-    }
 
-    /*
-     * Clean up any synchronization objects in the thread local storage.
-     */
+	/*
+	 * Clean up any synchronization objects in the thread local storage.
+	 */
 
 #ifdef __CYGWIN__
-    DestroyWindow(tsdPtr->hwnd);
-    CloseHandle(tsdPtr->event);
+	CloseHandle(tsdPtr->event);
 #else /* __CYGWIN__ */
-    pthread_cond_destroy(&tsdPtr->waitCV);
+	Tcl_ConditionFinalize(&(tsdPtr->waitCV));
 #endif /* __CYGWIN__ */
-    tsdPtr->waitCVinitialized = 0;
 
-    pthread_mutex_unlock(&notifierInitMutex);
+	Tcl_MutexUnlock(&notifierMutex);
 #endif /* TCL_THREADS */
+    }
 }
 
 /*
@@ -486,18 +449,23 @@ void
 Tcl_AlertNotifier(
     ClientData clientData)
 {
+    if (tclNotifierHooks.alertNotifierProc) {
+	tclNotifierHooks.alertNotifierProc(clientData);
+	return;
+    } else {
 #ifdef TCL_THREADS
-    ThreadSpecificData *tsdPtr = (ThreadSpecificData *) clientData;
+	ThreadSpecificData *tsdPtr = clientData;
 
-    pthread_mutex_lock(&notifierMutex);
-    tsdPtr->eventReady = 1;
-#ifdef __CYGWIN__
-    PostMessageW(tsdPtr->hwnd, 1024, 0, 0);
-#else /* __CYGWIN__ */
-    pthread_cond_broadcast(&tsdPtr->waitCV);
-#endif /* __CYGWIN__ */
-    pthread_mutex_unlock(&notifierMutex);
+	Tcl_MutexLock(&notifierMutex);
+	tsdPtr->eventReady = 1;
+#   ifdef __CYGWIN__
+	PostMessageW(tsdPtr->hwnd, 1024, 0, 0);
+#   else
+	Tcl_ConditionNotify(&tsdPtr->waitCV);
+#   endif /* __CYGWIN__ */
+	Tcl_MutexUnlock(&notifierMutex);
 #endif /* TCL_THREADS */
+    }
 }
 
 /*
@@ -520,16 +488,17 @@ Tcl_AlertNotifier(
 
 void
 Tcl_SetTimer(
-    Tcl_Time *timePtr)		/* Timeout value, may be NULL. */
+    const Tcl_Time *timePtr)		/* Timeout value, may be NULL. */
 {
-    /*
-     * The interval timer doesn't do anything in this implementation,
-     * because the only event loop is via Tcl_DoOneEvent, which passes
-     * timeout values to Tcl_WaitForEvent.
-     */
-
-    if (tclStubs.tcl_SetTimer != tclOriginalNotifier.setTimerProc) {
-	tclStubs.tcl_SetTimer(timePtr);
+    if (tclNotifierHooks.setTimerProc) {
+	tclNotifierHooks.setTimerProc(timePtr);
+	return;
+    } else {
+	/*
+	 * The interval timer doesn't do anything in this implementation,
+	 * because the only event loop is via Tcl_DoOneEvent, which passes
+	 * timeout values to Tcl_WaitForEvent.
+	 */
     }
 }
 
@@ -554,11 +523,12 @@ Tcl_ServiceModeHook(
     int mode)			/* Either TCL_SERVICE_ALL, or
 				 * TCL_SERVICE_NONE. */
 {
-#if TCL_THREADS
-    if (mode == TCL_SERVICE_ALL) {
-	StartNotifierThread("Tcl_ServiceModeHook");
+    if (tclNotifierHooks.serviceModeHookProc) {
+	tclNotifierHooks.serviceModeHookProc(mode);
+	return;
+    } else {
+	/* Does nothing in this implementation. */
     }
-#endif
 }
 
 /*
@@ -588,53 +558,52 @@ Tcl_CreateFileHandler(
 				 * event. */
     ClientData clientData)	/* Arbitrary data to pass to proc. */
 {
-    ThreadSpecificData *tsdPtr = TCL_TSD_INIT(&dataKey);
-    FileHandler *filePtr;
-
-    if (tclStubs.tcl_CreateFileHandler !=
-	    tclOriginalNotifier.createFileHandlerProc) {
-	tclStubs.tcl_CreateFileHandler(fd, mask, proc, clientData);
+    if (tclNotifierHooks.createFileHandlerProc) {
+	tclNotifierHooks.createFileHandlerProc(fd, mask, proc, clientData);
 	return;
-    }
+    } else {
+	ThreadSpecificData *tsdPtr = TCL_TSD_INIT(&dataKey);
+	FileHandler *filePtr;
 
-    for (filePtr = tsdPtr->firstFileHandlerPtr; filePtr != NULL;
-	    filePtr = filePtr->nextPtr) {
-	if (filePtr->fd == fd) {
-	    break;
+	for (filePtr = tsdPtr->firstFileHandlerPtr; filePtr != NULL;
+		filePtr = filePtr->nextPtr) {
+	    if (filePtr->fd == fd) {
+		break;
+	    }
 	}
-    }
-    if (filePtr == NULL) {
-	filePtr = (FileHandler*) ckalloc(sizeof(FileHandler));
-	filePtr->fd = fd;
-	filePtr->readyMask = 0;
-	filePtr->nextPtr = tsdPtr->firstFileHandlerPtr;
-	tsdPtr->firstFileHandlerPtr = filePtr;
-    }
-    filePtr->proc = proc;
-    filePtr->clientData = clientData;
-    filePtr->mask = mask;
+	if (filePtr == NULL) {
+	    filePtr = ckalloc(sizeof(FileHandler));
+	    filePtr->fd = fd;
+	    filePtr->readyMask = 0;
+	    filePtr->nextPtr = tsdPtr->firstFileHandlerPtr;
+	    tsdPtr->firstFileHandlerPtr = filePtr;
+	}
+	filePtr->proc = proc;
+	filePtr->clientData = clientData;
+	filePtr->mask = mask;
 
-    /*
-     * Update the check masks for this file.
-     */
+	/*
+	 * Update the check masks for this file.
+	 */
 
-    if (mask & TCL_READABLE) {
-	FD_SET(fd, &tsdPtr->checkMasks.readable);
-    } else {
-	FD_CLR(fd, &tsdPtr->checkMasks.readable);
-    }
-    if (mask & TCL_WRITABLE) {
-	FD_SET(fd, &tsdPtr->checkMasks.writable);
-    } else {
-	FD_CLR(fd, &tsdPtr->checkMasks.writable);
-    }
-    if (mask & TCL_EXCEPTION) {
-	FD_SET(fd, &tsdPtr->checkMasks.exception);
-    } else {
-	FD_CLR(fd, &tsdPtr->checkMasks.exception);
-    }
-    if (tsdPtr->numFdBits <= fd) {
-	tsdPtr->numFdBits = fd+1;
+	if (mask & TCL_READABLE) {
+	    FD_SET(fd, &tsdPtr->checkMasks.readable);
+	} else {
+	    FD_CLR(fd, &tsdPtr->checkMasks.readable);
+	}
+	if (mask & TCL_WRITABLE) {
+	    FD_SET(fd, &tsdPtr->checkMasks.writable);
+	} else {
+	    FD_CLR(fd, &tsdPtr->checkMasks.writable);
+	}
+	if (mask & TCL_EXCEPTION) {
+	    FD_SET(fd, &tsdPtr->checkMasks.exception);
+	} else {
+	    FD_CLR(fd, &tsdPtr->checkMasks.exception);
+	}
+	if (tsdPtr->numFdBits <= fd) {
+	    tsdPtr->numFdBits = fd+1;
+	}
     }
 }
 
@@ -659,72 +628,71 @@ Tcl_DeleteFileHandler(
     int fd)			/* Stream id for which to remove callback
 				 * function. */
 {
-    FileHandler *filePtr, *prevPtr;
-    int i;
-    ThreadSpecificData *tsdPtr = TCL_TSD_INIT(&dataKey);
-
-    if (tclStubs.tcl_DeleteFileHandler !=
-	    tclOriginalNotifier.deleteFileHandlerProc) {
-	tclStubs.tcl_DeleteFileHandler(fd);
+    if (tclNotifierHooks.deleteFileHandlerProc) {
+	tclNotifierHooks.deleteFileHandlerProc(fd);
 	return;
-    }
+    } else {
+	FileHandler *filePtr, *prevPtr;
+	int i;
+	ThreadSpecificData *tsdPtr = TCL_TSD_INIT(&dataKey);
 
-    /*
-     * Find the entry for the given file (and return if there isn't one).
-     */
+	/*
+	 * Find the entry for the given file (and return if there isn't one).
+	 */
 
-    for (prevPtr = NULL, filePtr = tsdPtr->firstFileHandlerPtr; ;
-	prevPtr = filePtr, filePtr = filePtr->nextPtr) {
-	if (filePtr == NULL) {
-	    return;
-	}
-	if (filePtr->fd == fd) {
-	    break;
-	}
-    }
-
-    /*
-     * Update the check masks for this file.
-     */
-
-    if (filePtr->mask & TCL_READABLE) {
-	FD_CLR(fd, &tsdPtr->checkMasks.readable);
-    }
-    if (filePtr->mask & TCL_WRITABLE) {
-	FD_CLR(fd, &tsdPtr->checkMasks.writable);
-    }
-    if (filePtr->mask & TCL_EXCEPTION) {
-	FD_CLR(fd, &tsdPtr->checkMasks.exception);
-    }
-
-    /*
-     * Find current max fd.
-     */
-
-    if (fd+1 == tsdPtr->numFdBits) {
-	int numFdBits = 0;
-
-	for (i = fd-1; i >= 0; i--) {
-	    if (FD_ISSET(i, &tsdPtr->checkMasks.readable)
-		    || FD_ISSET(i, &tsdPtr->checkMasks.writable)
-		    || FD_ISSET(i, &tsdPtr->checkMasks.exception)) {
-		numFdBits = i+1;
+	for (prevPtr = NULL, filePtr = tsdPtr->firstFileHandlerPtr; ;
+		prevPtr = filePtr, filePtr = filePtr->nextPtr) {
+	    if (filePtr == NULL) {
+		return;
+	    }
+	    if (filePtr->fd == fd) {
 		break;
 	    }
 	}
-	tsdPtr->numFdBits = numFdBits;
-    }
 
-    /*
-     * Clean up information in the callback record.
-     */
+	/*
+	 * Update the check masks for this file.
+	 */
 
-    if (prevPtr == NULL) {
-	tsdPtr->firstFileHandlerPtr = filePtr->nextPtr;
-    } else {
-	prevPtr->nextPtr = filePtr->nextPtr;
+	if (filePtr->mask & TCL_READABLE) {
+	    FD_CLR(fd, &tsdPtr->checkMasks.readable);
+	}
+	if (filePtr->mask & TCL_WRITABLE) {
+	    FD_CLR(fd, &tsdPtr->checkMasks.writable);
+	}
+	if (filePtr->mask & TCL_EXCEPTION) {
+	    FD_CLR(fd, &tsdPtr->checkMasks.exception);
+	}
+
+	/*
+	 * Find current max fd.
+	 */
+
+	if (fd+1 == tsdPtr->numFdBits) {
+	    int numFdBits = 0;
+
+	    for (i = fd-1; i >= 0; i--) {
+		if (FD_ISSET(i, &tsdPtr->checkMasks.readable)
+			|| FD_ISSET(i, &tsdPtr->checkMasks.writable)
+			|| FD_ISSET(i, &tsdPtr->checkMasks.exception)) {
+		    numFdBits = i+1;
+		    break;
+		}
+	    }
+	    tsdPtr->numFdBits = numFdBits;
+	}
+
+	/*
+	 * Clean up information in the callback record.
+	 */
+
+	if (prevPtr == NULL) {
+	    tsdPtr->firstFileHandlerPtr = filePtr->nextPtr;
+	} else {
+	    prevPtr->nextPtr = filePtr->nextPtr;
+	}
+	ckfree(filePtr);
     }
-    ckfree((char *) filePtr);
 }
 
 /*
@@ -772,7 +740,6 @@ FileHandlerEventProc(
      */
 
     tsdPtr = TCL_TSD_INIT(&dataKey);
-
     for (filePtr = tsdPtr->firstFileHandlerPtr; filePtr != NULL;
 	    filePtr = filePtr->nextPtr) {
 	if (filePtr->fd != fileEvPtr->fd) {
@@ -846,264 +813,275 @@ NotifierProc(
 
 int
 Tcl_WaitForEvent(
-    Tcl_Time *timePtr)		/* Maximum block time, or NULL. */
+    const Tcl_Time *timePtr)		/* Maximum block time, or NULL. */
 {
-    FileHandler *filePtr;
-    int mask;
-    Tcl_Time vTime;
+    if (tclNotifierHooks.waitForEventProc) {
+	return tclNotifierHooks.waitForEventProc(timePtr);
+    } else {
+	FileHandler *filePtr;
+	int mask;
+	Tcl_Time vTime;
 #ifdef TCL_THREADS
-    int waitForFiles;
-#ifdef __CYGWIN__
-    MSG msg;
-#endif /* __CYGWIN__ */
+	int waitForFiles;
+#   ifdef __CYGWIN__
+	MSG msg;
+#   endif /* __CYGWIN__ */
 #else
-    /*
-     * Impl. notes: timeout & timeoutPtr are used if, and only if threads
-     * are not enabled. They are the arguments for the regular select()
-     * used when the core is not thread-enabled.
-     */
+	/*
+	 * Impl. notes: timeout & timeoutPtr are used if, and only if threads
+	 * are not enabled. They are the arguments for the regular select()
+	 * used when the core is not thread-enabled.
+	 */
 
-    struct timeval timeout, *timeoutPtr;
-    int numFound;
+	struct timeval timeout, *timeoutPtr;
+	int numFound;
 #endif /* TCL_THREADS */
-    ThreadSpecificData *tsdPtr = TCL_TSD_INIT(&dataKey);
+	ThreadSpecificData *tsdPtr = TCL_TSD_INIT(&dataKey);
 
-    if (tclStubs.tcl_WaitForEvent != tclOriginalNotifier.waitForEventProc) {
-	return tclStubs.tcl_WaitForEvent(timePtr);
-    }
-
-    /*
-     * Set up the timeout structure. Note that if there are no events to
-     * check for, we return with a negative result rather than blocking
-     * forever.
-     */
-
-    if (timePtr != NULL) {
 	/*
-     * TIP #233 (Virtualized Time). Is virtual time in effect? And do
-     * we actually have something to scale? If yes to both then we
-     * call the handler to do this scaling.
+	 * Set up the timeout structure. Note that if there are no events to
+	 * check for, we return with a negative result rather than blocking
+	 * forever.
 	 */
 
-	if (timePtr->sec != 0 || timePtr->usec != 0) {
-	    vTime = *timePtr;
-	    tclScaleTimeProcPtr(&vTime, tclTimeClientData);
-	    timePtr = &vTime;
-	}
+	if (timePtr != NULL) {
+	    /*
+	     * TIP #233 (Virtualized Time). Is virtual time in effect? And do
+	     * we actually have something to scale? If yes to both then we
+	     * call the handler to do this scaling.
+	     */
+
+	    if (timePtr->sec != 0 || timePtr->usec != 0) {
+		vTime = *timePtr;
+		tclScaleTimeProcPtr(&vTime, tclTimeClientData);
+		timePtr = &vTime;
+	    }
 #ifndef TCL_THREADS
-	timeout.tv_sec = timePtr->sec;
-	timeout.tv_usec = timePtr->usec;
-	timeoutPtr = &timeout;
-    } else if (tsdPtr->numFdBits == 0) {
-	/*
-	 * If there are no threads, no timeout, and no fds registered,
-	 * then there are no events possible and we must avoid deadlock.
-	 * Note that this is not entirely correct because there might be a
-	 * signal that could interrupt the select call, but we don't
-	 * handle that case if we aren't using threads.
-	 */
+	    timeout.tv_sec = timePtr->sec;
+	    timeout.tv_usec = timePtr->usec;
+	    timeoutPtr = &timeout;
+	} else if (tsdPtr->numFdBits == 0) {
+	    /*
+	     * If there are no threads, no timeout, and no fds registered,
+	     * then there are no events possible and we must avoid deadlock.
+	     * Note that this is not entirely correct because there might be a
+	     * signal that could interrupt the select call, but we don't
+	     * handle that case if we aren't using threads.
+	     */
 
-	return -1;
-    } else {
-	timeoutPtr = NULL;
+	    return -1;
+	} else {
+	    timeoutPtr = NULL;
 #endif /* !TCL_THREADS */
-    }
+	}
 
 #ifdef TCL_THREADS
-    /*
-     * Start notifier thread and place this thread on the list of
-     * interested threads, signal the notifier thread, and wait for a
-     * response or a timeout.
-     */
-    StartNotifierThread("Tcl_WaitForEvent");
+	/*
+	 * Place this thread on the list of interested threads, signal the
+	 * notifier thread, and wait for a response or a timeout.
+	 */
 
-    pthread_mutex_lock(&notifierMutex);
+#ifdef __CYGWIN__
+	if (!tsdPtr->hwnd) {
+	    WNDCLASS class;
 
-    if (timePtr != NULL && timePtr->sec == 0 && (timePtr->usec == 0
+	    class.style = 0;
+	    class.cbClsExtra = 0;
+	    class.cbWndExtra = 0;
+	    class.hInstance = TclWinGetTclInstance();
+	    class.hbrBackground = NULL;
+	    class.lpszMenuName = NULL;
+	    class.lpszClassName = L"TclNotifier";
+	    class.lpfnWndProc = NotifierProc;
+	    class.hIcon = NULL;
+	    class.hCursor = NULL;
+
+	    RegisterClassW(&class);
+	    tsdPtr->hwnd = CreateWindowExW(NULL, class.lpszClassName,
+		    class.lpszClassName, 0, 0, 0, 0, 0, NULL, NULL,
+		    TclWinGetTclInstance(), NULL);
+	    tsdPtr->event = CreateEventW(NULL, 1 /* manual */,
+		    0 /* !signaled */, NULL);
+	}
+#endif /* __CYGWIN */
+
+	Tcl_MutexLock(&notifierMutex);
+
+	if (timePtr != NULL && timePtr->sec == 0 && (timePtr->usec == 0
 #if defined(__APPLE__) && defined(__LP64__)
-	    /*
-	     * On 64-bit Darwin, pthread_cond_timedwait() appears to have
-	     * a bug that causes it to wait forever when passed an
-	     * absolute time which has already been exceeded by the system
-	     * time; as a workaround, when given a very brief timeout,
-	     * just do a poll. [Bug 1457797]
-	     */
-	    || timePtr->usec < 10
+		/*
+		 * On 64-bit Darwin, pthread_cond_timedwait() appears to have
+		 * a bug that causes it to wait forever when passed an
+		 * absolute time which has already been exceeded by the system
+		 * time; as a workaround, when given a very brief timeout,
+		 * just do a poll. [Bug 1457797]
+		 */
+		|| timePtr->usec < 10
 #endif /* __APPLE__ && __LP64__ */
-	    )) {
-	/*
-	 * Cannot emulate a polling select with a polling condition
-	 * variable. Instead, pretend to wait for files and tell the
-	 * notifier thread what we are doing. The notifier thread makes
-	 * sure it goes through select with its select mask in the same
-	 * state as ours currently is. We block until that happens.
-	 */
+		)) {
+	    /*
+	     * Cannot emulate a polling select with a polling condition
+	     * variable. Instead, pretend to wait for files and tell the
+	     * notifier thread what we are doing. The notifier thread makes
+	     * sure it goes through select with its select mask in the same
+	     * state as ours currently is. We block until that happens.
+	     */
 
-	waitForFiles = 1;
-	tsdPtr->pollState = POLL_WANT;
-	timePtr = NULL;
-    } else {
-	waitForFiles = (tsdPtr->numFdBits > 0);
-	tsdPtr->pollState = 0;
-    }
-
-    if (waitForFiles) {
-	/*
-	 * Add the ThreadSpecificData structure of this thread to the list
-	 * of ThreadSpecificData structures of all threads that are
-	 * waiting on file events.
-	 */
-
-	tsdPtr->nextPtr = waitingListPtr;
-	if (waitingListPtr) {
-	    waitingListPtr->prevPtr = tsdPtr;
+	    waitForFiles = 1;
+	    tsdPtr->pollState = POLL_WANT;
+	    timePtr = NULL;
+	} else {
+	    waitForFiles = (tsdPtr->numFdBits > 0);
+	    tsdPtr->pollState = 0;
 	}
-	tsdPtr->prevPtr = 0;
-	waitingListPtr = tsdPtr;
-	tsdPtr->onList = 1;
 
-	if ((write(triggerPipe, "", 1) == -1) && (errno != EAGAIN)) {
-	    Tcl_Panic("Tcl_WaitForEvent: %s",
-		    "unable to write to triggerPipe");
-	}
-    }
+	if (waitForFiles) {
+	    /*
+	     * Add the ThreadSpecificData structure of this thread to the list
+	     * of ThreadSpecificData structures of all threads that are
+	     * waiting on file events.
+	     */
 
-    FD_ZERO(&tsdPtr->readyMasks.readable);
-    FD_ZERO(&tsdPtr->readyMasks.writable);
-    FD_ZERO(&tsdPtr->readyMasks.exception);
+	    tsdPtr->nextPtr = waitingListPtr;
+	    if (waitingListPtr) {
+		waitingListPtr->prevPtr = tsdPtr;
+	    }
+	    tsdPtr->prevPtr = 0;
+	    waitingListPtr = tsdPtr;
+	    tsdPtr->onList = 1;
 
-    if (!tsdPtr->eventReady) {
-#ifdef __CYGWIN__
-	if (!PeekMessageW(&msg, NULL, 0, 0, 0)) {
-	    DWORD timeout;
-
-	    if (timePtr) {
-		timeout = timePtr->sec * 1000 + timePtr->usec / 1000;
-	    } else {
-		timeout = 0xFFFFFFFF;
+	    if ((write(triggerPipe, "", 1) == -1) && (errno != EAGAIN)) {
+		Tcl_Panic("Tcl_WaitForEvent: %s",
+			"unable to write to triggerPipe");
 	    }
-	    pthread_mutex_unlock(&notifierMutex);
-	    MsgWaitForMultipleObjects(1, &tsdPtr->event, 0, timeout, 1279);
-	    pthread_mutex_lock(&notifierMutex);
 	}
-#else
-	if (timePtr != NULL) {
-	    Tcl_Time now;
-	    struct timespec ptime;
 
-	    Tcl_GetTime(&now);
-	    ptime.tv_sec = timePtr->sec + now.sec + (timePtr->usec + now.usec) / 1000000;
-	    ptime.tv_nsec = 1000 * ((timePtr->usec + now.usec) % 1000000);
-
-	    pthread_cond_timedwait(&tsdPtr->waitCV, &notifierMutex, &ptime);
-	} else {
-	    pthread_cond_wait(&tsdPtr->waitCV, &notifierMutex);
-	}
-#endif /* __CYGWIN__ */
-    }
-    tsdPtr->eventReady = 0;
-
-#ifdef __CYGWIN__
-    while (PeekMessageW(&msg, NULL, 0, 0, 0)) {
-	/*
-	 * Retrieve and dispatch the message.
-	 */
-
-	DWORD result = GetMessageW(&msg, NULL, 0, 0);
-
-	if (result == 0) {
-	    PostQuitMessage(msg.wParam);
-	    /* What to do here? */
-	} else if (result != (DWORD) -1) {
-	    TranslateMessage(&msg);
-	    DispatchMessageW(&msg);
-	}
-    }
-    ResetEvent(tsdPtr->event);
-#endif /* __CYGWIN__ */
-
-    if (waitForFiles && tsdPtr->onList) {
-	/*
-	 * Remove the ThreadSpecificData structure of this thread from the
-	 * waiting list. Alert the notifier thread to recompute its select
-	 * masks - skipping this caused a hang when trying to close a pipe
-	 * which the notifier thread was still doing a select on.
-	 */
-
-	if (tsdPtr->prevPtr) {
-	    tsdPtr->prevPtr->nextPtr = tsdPtr->nextPtr;
-	} else {
-	    waitingListPtr = tsdPtr->nextPtr;
-	}
-	if (tsdPtr->nextPtr) {
-	    tsdPtr->nextPtr->prevPtr = tsdPtr->prevPtr;
-	}
-	tsdPtr->nextPtr = tsdPtr->prevPtr = NULL;
-	tsdPtr->onList = 0;
-	if ((write(triggerPipe, "", 1) == -1) && (errno != EAGAIN)) {
-	    Tcl_Panic("Tcl_WaitForEvent: %s",
-		    "unable to write to triggerPipe");
-	}
-    }
-
-#else
-    tsdPtr->readyMasks = tsdPtr->checkMasks;
-    numFound = select(tsdPtr->numFdBits, &tsdPtr->readyMasks.readable,
-	    &tsdPtr->readyMasks.writable, &tsdPtr->readyMasks.exception,
-	    timeoutPtr);
-
-    /*
-     * Some systems don't clear the masks after an error, so we have to do
-     * it here.
-     */
-
-    if (numFound == -1) {
 	FD_ZERO(&tsdPtr->readyMasks.readable);
 	FD_ZERO(&tsdPtr->readyMasks.writable);
 	FD_ZERO(&tsdPtr->readyMasks.exception);
-    }
-#endif /* TCL_THREADS */
 
-    /*
-     * Queue all detected file events before returning.
-     */
+	if (!tsdPtr->eventReady) {
+#ifdef __CYGWIN__
+	    if (!PeekMessageW(&msg, NULL, 0, 0, 0)) {
+		DWORD timeout;
 
-    for (filePtr = tsdPtr->firstFileHandlerPtr; (filePtr != NULL);
-	    filePtr = filePtr->nextPtr) {
-	mask = 0;
-	if (FD_ISSET(filePtr->fd, &tsdPtr->readyMasks.readable)) {
-	    mask |= TCL_READABLE;
+		if (timePtr) {
+		    timeout = timePtr->sec * 1000 + timePtr->usec / 1000;
+		} else {
+		    timeout = 0xFFFFFFFF;
+		}
+		Tcl_MutexUnlock(&notifierMutex);
+		MsgWaitForMultipleObjects(1, &tsdPtr->event, 0, timeout, 1279);
+		Tcl_MutexLock(&notifierMutex);
+	    }
+#else
+	    Tcl_ConditionWait(&tsdPtr->waitCV, &notifierMutex, timePtr);
+#endif /* __CYGWIN__ */
 	}
-	if (FD_ISSET(filePtr->fd, &tsdPtr->readyMasks.writable)) {
-	    mask |= TCL_WRITABLE;
+	tsdPtr->eventReady = 0;
+
+#ifdef __CYGWIN__
+	while (PeekMessageW(&msg, NULL, 0, 0, 0)) {
+	    /*
+	     * Retrieve and dispatch the message.
+	     */
+
+	    DWORD result = GetMessageW(&msg, NULL, 0, 0);
+
+	    if (result == 0) {
+		PostQuitMessage(msg.wParam);
+		/* What to do here? */
+	    } else if (result != (DWORD) -1) {
+		TranslateMessage(&msg);
+		DispatchMessageW(&msg);
+	    }
 	}
-	if (FD_ISSET(filePtr->fd, &tsdPtr->readyMasks.exception)) {
-	    mask |= TCL_EXCEPTION;
+	ResetEvent(tsdPtr->event);
+#endif /* __CYGWIN__ */
+
+	if (waitForFiles && tsdPtr->onList) {
+	    /*
+	     * Remove the ThreadSpecificData structure of this thread from the
+	     * waiting list. Alert the notifier thread to recompute its select
+	     * masks - skipping this caused a hang when trying to close a pipe
+	     * which the notifier thread was still doing a select on.
+	     */
+
+	    if (tsdPtr->prevPtr) {
+		tsdPtr->prevPtr->nextPtr = tsdPtr->nextPtr;
+	    } else {
+		waitingListPtr = tsdPtr->nextPtr;
+	    }
+	    if (tsdPtr->nextPtr) {
+		tsdPtr->nextPtr->prevPtr = tsdPtr->prevPtr;
+	    }
+	    tsdPtr->nextPtr = tsdPtr->prevPtr = NULL;
+	    tsdPtr->onList = 0;
+	    if ((write(triggerPipe, "", 1) == -1) && (errno != EAGAIN)) {
+		Tcl_Panic("Tcl_WaitForEvent: %s",
+			"unable to write to triggerPipe");
+	    }
 	}
 
-	if (!mask) {
-	    continue;
-	}
+#else
+	tsdPtr->readyMasks = tsdPtr->checkMasks;
+	numFound = select(tsdPtr->numFdBits, &tsdPtr->readyMasks.readable,
+		&tsdPtr->readyMasks.writable, &tsdPtr->readyMasks.exception,
+		timeoutPtr);
 
 	/*
-	 * Don't bother to queue an event if the mask was previously
-	 * non-zero since an event must still be on the queue.
+	 * Some systems don't clear the masks after an error, so we have to do
+	 * it here.
 	 */
 
-	if (filePtr->readyMask == 0) {
-	    FileHandlerEvent *fileEvPtr =
-		    (FileHandlerEvent *) ckalloc(sizeof(FileHandlerEvent));
-
-	    fileEvPtr->header.proc = FileHandlerEventProc;
-	    fileEvPtr->fd = filePtr->fd;
-	    Tcl_QueueEvent((Tcl_Event *) fileEvPtr, TCL_QUEUE_TAIL);
+	if (numFound == -1) {
+	    FD_ZERO(&tsdPtr->readyMasks.readable);
+	    FD_ZERO(&tsdPtr->readyMasks.writable);
+	    FD_ZERO(&tsdPtr->readyMasks.exception);
 	}
-	filePtr->readyMask = mask;
-    }
-#ifdef TCL_THREADS
-    pthread_mutex_unlock(&notifierMutex);
 #endif /* TCL_THREADS */
-    return 0;
+
+	/*
+	 * Queue all detected file events before returning.
+	 */
+
+	for (filePtr = tsdPtr->firstFileHandlerPtr; (filePtr != NULL);
+		filePtr = filePtr->nextPtr) {
+	    mask = 0;
+	    if (FD_ISSET(filePtr->fd, &tsdPtr->readyMasks.readable)) {
+		mask |= TCL_READABLE;
+	    }
+	    if (FD_ISSET(filePtr->fd, &tsdPtr->readyMasks.writable)) {
+		mask |= TCL_WRITABLE;
+	    }
+	    if (FD_ISSET(filePtr->fd, &tsdPtr->readyMasks.exception)) {
+		mask |= TCL_EXCEPTION;
+	    }
+
+	    if (!mask) {
+		continue;
+	    }
+
+	    /*
+	     * Don't bother to queue an event if the mask was previously
+	     * non-zero since an event must still be on the queue.
+	     */
+
+	    if (filePtr->readyMask == 0) {
+		FileHandlerEvent *fileEvPtr =
+			ckalloc(sizeof(FileHandlerEvent));
+
+		fileEvPtr->header.proc = FileHandlerEventProc;
+		fileEvPtr->fd = filePtr->fd;
+		Tcl_QueueEvent((Tcl_Event *) fileEvPtr, TCL_QUEUE_TAIL);
+	    }
+	    filePtr->readyMask = mask;
+	}
+#ifdef TCL_THREADS
+	Tcl_MutexUnlock(&notifierMutex);
+#endif /* TCL_THREADS */
+	return 0;
+    }
 }
 
 #ifdef TCL_THREADS
@@ -1172,15 +1150,15 @@ NotifierThreadProc(
      * Install the write end of the pipe into the global variable.
      */
 
-    pthread_mutex_lock(&notifierMutex);
+    Tcl_MutexLock(&notifierMutex);
     triggerPipe = fds[1];
 
     /*
      * Signal any threads that are waiting.
      */
 
-    pthread_cond_broadcast(&notifierCV);
-    pthread_mutex_unlock(&notifierMutex);
+    Tcl_ConditionNotify(&notifierCV);
+    Tcl_MutexUnlock(&notifierMutex);
 
     /*
      * Look for file events and report them to interested threads.
@@ -1196,7 +1174,7 @@ NotifierThreadProc(
 	 * notifiers.
 	 */
 
-	pthread_mutex_lock(&notifierMutex);
+	Tcl_MutexLock(&notifierMutex);
 	timePtr = NULL;
 	for (tsdPtr = waitingListPtr; tsdPtr; tsdPtr = tsdPtr->nextPtr) {
 	    for (i = tsdPtr->numFdBits-1; i >= 0; --i) {
@@ -1223,7 +1201,7 @@ NotifierThreadProc(
 		timePtr = &poll;
 	    }
 	}
-	pthread_mutex_unlock(&notifierMutex);
+	Tcl_MutexUnlock(&notifierMutex);
 
 	/*
 	 * Set up the select mask to include the receive pipe.
@@ -1247,7 +1225,7 @@ NotifierThreadProc(
 	 * Alert any threads that are waiting on a ready file descriptor.
 	 */
 
-	pthread_mutex_lock(&notifierMutex);
+	Tcl_MutexLock(&notifierMutex);
 	for (tsdPtr = waitingListPtr; tsdPtr; tsdPtr = tsdPtr->nextPtr) {
 	    found = 0;
 
@@ -1293,12 +1271,12 @@ NotifierThreadProc(
 		}
 #ifdef __CYGWIN__
 		PostMessageW(tsdPtr->hwnd, 1024, 0, 0);
-#else /* __CYGWIN__ */
-		pthread_cond_broadcast(&tsdPtr->waitCV);
+#else
+		Tcl_ConditionNotify(&tsdPtr->waitCV);
 #endif /* __CYGWIN__ */
 	    }
 	}
-	pthread_mutex_unlock(&notifierMutex);
+	Tcl_MutexUnlock(&notifierMutex);
 
 	/*
 	 * Consume the next byte from the notifier pipe if the pipe was
@@ -1327,15 +1305,15 @@ NotifierThreadProc(
      */
 
     close(receivePipe);
-    pthread_mutex_lock(&notifierMutex);
+    Tcl_MutexLock(&notifierMutex);
     triggerPipe = -1;
-    pthread_cond_broadcast(&notifierCV);
-    pthread_mutex_unlock(&notifierMutex);
+    Tcl_ConditionNotify(&notifierCV);
+    Tcl_MutexUnlock(&notifierMutex);
 
     TclpThreadExit(0);
 }
 
-#if defined(HAVE_PTHREAD_ATFORK)
+#ifdef HAVE_PTHREAD_ATFORK
 /*
  *----------------------------------------------------------------------
  *
@@ -1355,9 +1333,6 @@ NotifierThreadProc(
 static void
 AtForkPrepare(void)
 {
-#if RESET_ATFORK_MUTEX == 0
-    pthread_mutex_lock(&notifierInitMutex);
-#endif
 }
 
 /*
@@ -1379,9 +1354,6 @@ AtForkPrepare(void)
 static void
 AtForkParent(void)
 {
-#if RESET_ATFORK_MUTEX == 0
-    pthread_mutex_unlock(&notifierInitMutex);
-#endif
 }
 
 /*
@@ -1403,67 +1375,8 @@ AtForkParent(void)
 static void
 AtForkChild(void)
 {
-    if (notifierThreadRunning == 1) {
-	pthread_cond_destroy(&notifierCV);
-    }
-#if RESET_ATFORK_MUTEX == 0
-    pthread_mutex_unlock(&notifierInitMutex);
-#else
-    pthread_mutex_init(&notifierInitMutex, NULL);
-    pthread_mutex_init(&notifierMutex, NULL);
-#endif
-    pthread_cond_init(&notifierCV, NULL);
-
-    /*
-     * notifierThreadRunning == 1: thread is running, (there might be data in notifier lists)
-     * atForkInit == 0: InitNotifier was never called
-     * notifierCount != 0: unbalanced  InitNotifier() / FinalizeNotifier calls
-     * waitingListPtr != 0: there are threads currently waiting for events.
-     */
-
-    if (atForkInit == 1) {
-
-	notifierCount = 0;
-	if (notifierThreadRunning == 1) {
-	    ThreadSpecificData *tsdPtr = TCL_TSD_INIT(&dataKey);
-	    notifierThreadRunning = 0;
-
-	    close(triggerPipe);
-	    triggerPipe = -1;
-	    /*
-	     * The waitingListPtr might contain event info from multiple
-	     * threads, which are invalid here, so setting it to NULL is not
-	     * unreasonable.
-	     */
-	    waitingListPtr = NULL;
-
-	    /*
-	     * The tsdPtr from before the fork is copied as well.  But since
-	     * we are paranoic, we don't trust its condvar and reset it.
-	     */
-#ifdef __CYGWIN__
-	    DestroyWindow(tsdPtr->hwnd);
-	    tsdPtr->hwnd = CreateWindowExW(NULL, L"TclNotifier",
-		    L"TclNotifier", 0, 0, 0, 0, 0, NULL, NULL,
-		    TclWinGetTclInstance(), NULL);
-	    ResetEvent(tsdPtr->event);
-#else
-	    pthread_cond_destroy(&tsdPtr->waitCV);
-	    pthread_cond_init(&tsdPtr->waitCV, NULL);
-#endif
-	    /*
-	     * In case, we had multiple threads running before the fork,
-	     * make sure, we don't try to reach out to their thread local data.
-	     */
-	    tsdPtr->nextPtr = tsdPtr->prevPtr = NULL;
-
-	    /*
-	     * The list of registered event handlers at fork time is in
-	     * tsdPtr->firstFileHandlerPtr;
-	     */
-	}
-    }
-
+    notifierMutex = NULL;
+    notifierCV = NULL;
     Tcl_InitNotifier();
 }
 #endif /* HAVE_PTHREAD_ATFORK */
