@@ -192,7 +192,7 @@ static DWORD WINAPI	PipeReaderThread(LPVOID arg);
 static void		PipeSetupProc(ClientData clientData, int flags);
 static void		PipeWatchProc(ClientData instanceData, int mask);
 static DWORD WINAPI	PipeWriterThread(LPVOID arg);
-static int		TempFileName(WCHAR name[MAX_PATH]);
+static int		TempFileName(TCHAR name[MAX_PATH]);
 static int		WaitForRead(PipeInfo *infoPtr, int blocking);
 static void		PipeThreadActionProc(ClientData instanceData,
 			    int action);
@@ -202,7 +202,7 @@ static void		PipeThreadActionProc(ClientData instanceData,
  * I/O.
  */
 
-static Tcl_ChannelType pipeChannelType = {
+static const Tcl_ChannelType pipeChannelType = {
     "pipe",			/* Type name. */
     TCL_CHANNEL_VERSION_5,	/* v5 channel */
     TCL_CLOSE2PROC,		/* Close proc. */
@@ -219,7 +219,7 @@ static Tcl_ChannelType pipeChannelType = {
     NULL,			/* handler proc. */
     NULL,			/* wide seek proc */
     PipeThreadActionProc,	/* thread action proc */
-    NULL,                       /* truncate */
+    NULL                       /* truncate */
 };
 
 /*
@@ -404,7 +404,7 @@ PipeCheckProc(
 
 	if (needEvent) {
 	    infoPtr->flags |= PIPE_PENDING;
-	    evPtr = (PipeEvent *) ckalloc(sizeof(PipeEvent));
+	    evPtr = ckalloc(sizeof(PipeEvent));
 	    evPtr->header.proc = PipeEventProc;
 	    evPtr->infoPtr = infoPtr;
 	    Tcl_QueueEvent((Tcl_Event *) evPtr, TCL_QUEUE_TAIL);
@@ -435,7 +435,7 @@ TclWinMakeFile(
 {
     WinFile *filePtr;
 
-    filePtr = (WinFile *) ckalloc(sizeof(WinFile));
+    filePtr = ckalloc(sizeof(WinFile));
     filePtr->type = WIN_FILE;
     filePtr->handle = handle;
 
@@ -464,27 +464,18 @@ TclWinMakeFile(
 
 static int
 TempFileName(
-    WCHAR name[MAX_PATH])	/* Buffer in which name for temporary file
+    TCHAR name[MAX_PATH])	/* Buffer in which name for temporary file
 				 * gets stored. */
 {
-    TCHAR *prefix;
-
-    prefix = (tclWinProcs->useWide) ? (TCHAR *) L"TCL" : (TCHAR *) "TCL";
-    if ((*tclWinProcs->getTempPathProc)(MAX_PATH, name) != 0) {
-	if ((*tclWinProcs->getTempFileNameProc)((TCHAR *) name, prefix, 0,
-		name) != 0) {
+    const TCHAR *prefix = TEXT("TCL");
+    if (GetTempPath(MAX_PATH, name) != 0) {
+	if (GetTempFileName(name, prefix, 0, name) != 0) {
 	    return 1;
 	}
     }
-    if (tclWinProcs->useWide) {
-	((WCHAR *) name)[0] = '.';
-	((WCHAR *) name)[1] = '\0';
-    } else {
-	((char *) name)[0] = '.';
-	((char *) name)[1] = '\0';
-    }
-    return (*tclWinProcs->getTempFileNameProc)((TCHAR *) name, prefix, 0,
-	    name);
+    name[0] = '.';
+    name[1] = '\0';
+    return GetTempFileName(name, prefix, 0, name);
 }
 
 /*
@@ -596,7 +587,7 @@ TclpOpenFile(
 
     flags = 0;
     if (!(mode & O_CREAT)) {
-	flags = (*tclWinProcs->getFileAttributesProc)(nativePath);
+	flags = GetFileAttributes(nativePath);
 	if (flags == 0xFFFFFFFF) {
 	    flags = 0;
 	}
@@ -612,8 +603,8 @@ TclpOpenFile(
      * Now we get to create the file.
      */
 
-    handle = (*tclWinProcs->createFileProc)(nativePath, accessMode,
-	    shareMode, NULL, createMode, flags, NULL);
+    handle = CreateFile(nativePath, accessMode, shareMode,
+	    NULL, createMode, flags, NULL);
     Tcl_DStringFree(&ds);
 
     if (handle == INVALID_HANDLE_VALUE) {
@@ -660,7 +651,7 @@ TclFile
 TclpCreateTempFile(
     const char *contents)	/* String to write into temp file, or NULL. */
 {
-    WCHAR name[MAX_PATH];
+    TCHAR name[MAX_PATH];
     const char *native;
     Tcl_DString dstring;
     HANDLE handle;
@@ -669,7 +660,7 @@ TclpCreateTempFile(
 	return NULL;
     }
 
-    handle = (*tclWinProcs->createFileProc)((TCHAR *) name,
+    handle = CreateFile(name,
 	    GENERIC_READ | GENERIC_WRITE, 0, NULL, CREATE_ALWAYS,
 	    FILE_ATTRIBUTE_TEMPORARY|FILE_FLAG_DELETE_ON_CLOSE, NULL);
     if (handle == INVALID_HANDLE_VALUE) {
@@ -731,7 +722,7 @@ TclpCreateTempFile(
 
     TclWinConvertError(GetLastError());
     CloseHandle(handle);
-    (*tclWinProcs->deleteFileProc)((TCHAR *) name);
+    DeleteFile(name);
     return NULL;
 }
 
@@ -754,13 +745,13 @@ TclpCreateTempFile(
 Tcl_Obj *
 TclpTempFileName(void)
 {
-    WCHAR fileName[MAX_PATH];
+    TCHAR fileName[MAX_PATH];
 
     if (TempFileName(fileName) == 0) {
 	return NULL;
     }
 
-    return TclpNativeToNormalized((ClientData) fileName);
+    return TclpNativeToNormalized(fileName);
 }
 
 /*
@@ -836,7 +827,7 @@ TclpCloseFile(
 	    if (filePtr->handle != NULL &&
 		    CloseHandle(filePtr->handle) == FALSE) {
 		TclWinConvertError(GetLastError());
-		ckfree((char *) filePtr);
+		ckfree(filePtr);
 		return -1;
 	    }
 	}
@@ -846,7 +837,7 @@ TclpCloseFile(
 	Tcl_Panic("TclpCloseFile: unexpected file type");
     }
 
-    ckfree((char *) filePtr);
+    ckfree(filePtr);
     return 0;
 }
 
@@ -947,7 +938,7 @@ TclpCreateProcess(
 {
     int result, applType, createFlags;
     Tcl_DString cmdLine;	/* Complete command line (TCHAR). */
-    STARTUPINFOA startInfo;
+    STARTUPINFO startInfo;
     PROCESS_INFORMATION procInfo;
     SECURITY_ATTRIBUTES secAtts;
     HANDLE hProcess, h, inputHandle, outputHandle, errorHandle;
@@ -1037,8 +1028,9 @@ TclpCreateProcess(
     }
     if (startInfo.hStdInput == INVALID_HANDLE_VALUE) {
 	TclWinConvertError(GetLastError());
-	Tcl_AppendResult(interp, "couldn't duplicate input handle: ",
-		Tcl_PosixError(interp), (char *) NULL);
+	Tcl_SetObjResult(interp, Tcl_ObjPrintf(
+		"couldn't duplicate input handle: %s",
+		Tcl_PosixError(interp)));
 	goto end;
     }
 
@@ -1057,23 +1049,17 @@ TclpCreateProcess(
 	 * sink.
 	 */
 
-	if ((TclWinGetPlatformId() == VER_PLATFORM_WIN32_WINDOWS)
-		&& (applType == APPL_DOS)) {
-	    if (CreatePipe(&h, &startInfo.hStdOutput, &secAtts, 0) != FALSE) {
-		CloseHandle(h);
-	    }
-	} else {
-	    startInfo.hStdOutput = CreateFileA("NUL:", GENERIC_WRITE, 0,
-		    &secAtts, OPEN_ALWAYS, FILE_ATTRIBUTE_NORMAL, NULL);
-	}
+	startInfo.hStdOutput = CreateFile(TEXT("NUL:"), GENERIC_WRITE, 0,
+		&secAtts, OPEN_ALWAYS, FILE_ATTRIBUTE_NORMAL, NULL);
     } else {
 	DuplicateHandle(hProcess, outputHandle, hProcess,
 		&startInfo.hStdOutput, 0, TRUE, DUPLICATE_SAME_ACCESS);
     }
     if (startInfo.hStdOutput == INVALID_HANDLE_VALUE) {
 	TclWinConvertError(GetLastError());
-	Tcl_AppendResult(interp, "couldn't duplicate output handle: ",
-		Tcl_PosixError(interp), (char *) NULL);
+	Tcl_SetObjResult(interp, Tcl_ObjPrintf(
+		"couldn't duplicate output handle: %s",
+		Tcl_PosixError(interp)));
 	goto end;
     }
 
@@ -1083,7 +1069,7 @@ TclpCreateProcess(
 	 * sink.
 	 */
 
-	startInfo.hStdError = CreateFileA("NUL:", GENERIC_WRITE, 0,
+	startInfo.hStdError = CreateFile(TEXT("NUL:"), GENERIC_WRITE, 0,
 		&secAtts, OPEN_EXISTING, FILE_ATTRIBUTE_NORMAL, NULL);
     } else {
 	DuplicateHandle(hProcess, errorHandle, hProcess, &startInfo.hStdError,
@@ -1091,8 +1077,9 @@ TclpCreateProcess(
     }
     if (startInfo.hStdError == INVALID_HANDLE_VALUE) {
 	TclWinConvertError(GetLastError());
-	Tcl_AppendResult(interp, "couldn't duplicate error handle: ",
-		Tcl_PosixError(interp), (char *) NULL);
+	Tcl_SetObjResult(interp, Tcl_ObjPrintf(
+		"couldn't duplicate error handle: %s",
+		Tcl_PosixError(interp)));
 	goto end;
     }
 
@@ -1124,7 +1111,7 @@ TclpCreateProcess(
 	    startInfo.wShowWindow = SW_HIDE;
 	    startInfo.dwFlags |= STARTF_USESHOWWINDOW;
 	    createFlags = CREATE_NEW_CONSOLE;
-	    Tcl_DStringAppend(&cmdLine, "cmd.exe /c", -1);
+	    TclDStringAppendLiteral(&cmdLine, "cmd.exe /c");
 	} else {
 	    createFlags = DETACHED_PROCESS;
 	}
@@ -1136,82 +1123,12 @@ TclpCreateProcess(
 	}
 
 	if (applType == APPL_DOS) {
-	    /*
-	     * Under Windows 95, 16-bit DOS applications do not work well with
-	     * pipes:
-	     *
-	     * 1. EOF on a pipe between a detached 16-bit DOS application and
-	     * another application is not seen at the other end of the pipe,
-	     * so the listening process blocks forever on reads. This inablity
-	     * to detect EOF happens when either a 16-bit app or the 32-bit
-	     * app is the listener.
-	     *
-	     * 2. If a 16-bit DOS application (detached or not) blocks when
-	     * writing to a pipe, it will never wake up again, and it
-	     * eventually brings the whole system down around it.
-	     *
-	     * The 16-bit application is run as a normal process inside of a
-	     * hidden helper console app, and this helper may be run as a
-	     * detached process. If any of the stdio handles is a pipe, the
-	     * helper application accumulates information into temp files and
-	     * forwards it to or from the DOS application as appropriate.
-	     * This means that DOS apps must receive EOF from a stdin pipe
-	     * before they will actually begin, and must finish generating
-	     * stdout or stderr before the data will be sent to the next stage
-	     * of the pipe.
-	     *
-	     * The helper app should be located in the same directory as the
-	     * tcl dll.
-	     */
-	    Tcl_Obj *tclExePtr, *pipeDllPtr;
-	    char *start, *end;
-	    int i, fileExists;
-	    Tcl_DString pipeDll;
-
-	    if (createFlags != 0) {
-		startInfo.wShowWindow = SW_HIDE;
-		startInfo.dwFlags |= STARTF_USESHOWWINDOW;
-		createFlags = CREATE_NEW_CONSOLE;
-	    }
-
-	    Tcl_DStringInit(&pipeDll);
-	    Tcl_DStringAppend(&pipeDll, TCL_PIPE_DLL, -1);
-	    tclExePtr = TclGetObjNameOfExecutable();
-	    Tcl_IncrRefCount(tclExePtr);
-	    start = Tcl_GetStringFromObj(tclExePtr, &i);
-	    for (end = start + (i-1); end > start; end--) {
-		if (*end == '/') {
-		    break;
-		}
-	    }
-	    if (*end != '/') {
-		Tcl_AppendResult(interp, "no / in executable path name \"",
-			start, "\"", (char *) NULL);
-		Tcl_DecrRefCount(tclExePtr);
-		Tcl_DStringFree(&pipeDll);
-		goto end;
-	    }
-	    i = (end - start) + 1;
-	    pipeDllPtr = Tcl_NewStringObj(start, i);
-	    Tcl_AppendToObj(pipeDllPtr, Tcl_DStringValue(&pipeDll), -1);
-	    Tcl_IncrRefCount(pipeDllPtr);
-	    if (Tcl_FSConvertToPathType(interp, pipeDllPtr) != TCL_OK) {
-		Tcl_Panic("Tcl_FSConvertToPathType failed");
-	    }
-	    fileExists = (Tcl_FSAccess(pipeDllPtr, F_OK) == 0);
-	    if (!fileExists) {
-		Tcl_AppendResult(interp, "Tcl pipe dll \"",
-			Tcl_DStringValue(&pipeDll), "\" not found",
-			(char *) NULL);
-		Tcl_DecrRefCount(tclExePtr);
-		Tcl_DecrRefCount(pipeDllPtr);
-		Tcl_DStringFree(&pipeDll);
-		goto end;
-	    }
-	    Tcl_DStringAppend(&cmdLine, Tcl_DStringValue(&pipeDll), -1);
-	    Tcl_DecrRefCount(tclExePtr);
-	    Tcl_DecrRefCount(pipeDllPtr);
-	    Tcl_DStringFree(&pipeDll);
+	    Tcl_SetObjResult(interp, Tcl_NewStringObj(
+		    "DOS application process not supported on this platform",
+		    -1));
+	    Tcl_SetErrorCode(interp, "TCL", "OPERATION", "EXEC", "DOS_APP",
+		    NULL);
+	    goto end;
 	}
     }
 
@@ -1235,12 +1152,12 @@ TclpCreateProcess(
 
     BuildCommandLine(execPath, argc, argv, &cmdLine);
 
-    if ((*tclWinProcs->createProcessProc)(NULL,
-	    (TCHAR *) Tcl_DStringValue(&cmdLine), NULL, NULL, TRUE,
-	    (DWORD) createFlags, NULL, NULL, &startInfo, &procInfo) == 0) {
+    if (CreateProcess(NULL, (TCHAR *) Tcl_DStringValue(&cmdLine),
+	    NULL, NULL, TRUE, (DWORD) createFlags, NULL, NULL, &startInfo,
+	    &procInfo) == 0) {
 	TclWinConvertError(GetLastError());
-	Tcl_AppendResult(interp, "couldn't execute \"", argv[0],
-		"\": ", Tcl_PosixError(interp), (char *) NULL);
+	Tcl_SetObjResult(interp, Tcl_ObjPrintf("couldn't execute \"%s\": %s",
+		argv[0], Tcl_PosixError(interp)));
 	goto end;
     }
 
@@ -1368,7 +1285,7 @@ ApplicationType(
     IMAGE_DOS_HEADER header;
     Tcl_DString nameBuf, ds;
     const TCHAR *nativeName;
-    WCHAR nativeFullPath[MAX_PATH];
+    TCHAR nativeFullPath[MAX_PATH];
     static const char extensions[][5] = {"", ".com", ".exe", ".bat"};
 
     /*
@@ -1394,8 +1311,8 @@ ApplicationType(
 	Tcl_DStringAppend(&nameBuf, extensions[i], -1);
 	nativeName = Tcl_WinUtfToTChar(Tcl_DStringValue(&nameBuf),
 		Tcl_DStringLength(&nameBuf), &ds);
-	found = (*tclWinProcs->searchPathProc)(NULL, nativeName, NULL,
-		MAX_PATH, nativeFullPath, &rest);
+	found = SearchPath(NULL, nativeName, NULL, MAX_PATH,
+		nativeFullPath, &rest);
 	Tcl_DStringFree(&ds);
 	if (found == 0) {
 	    continue;
@@ -1406,11 +1323,11 @@ ApplicationType(
 	 * known type.
 	 */
 
-	attr = (*tclWinProcs->getFileAttributesProc)((TCHAR *) nativeFullPath);
+	attr = GetFileAttributes(nativeFullPath);
 	if ((attr == 0xffffffff) || (attr & FILE_ATTRIBUTE_DIRECTORY)) {
 	    continue;
 	}
-	strcpy(fullName, Tcl_WinTCharToUtf((TCHAR *) nativeFullPath, -1, &ds));
+	strcpy(fullName, Tcl_WinTCharToUtf(nativeFullPath, -1, &ds));
 	Tcl_DStringFree(&ds);
 
 	ext = strrchr(fullName, '.');
@@ -1419,7 +1336,7 @@ ApplicationType(
 	    break;
 	}
 
-	hFile = (*tclWinProcs->createFileProc)((TCHAR *) nativeFullPath,
+	hFile = CreateFile(nativeFullPath,
 		GENERIC_READ, FILE_SHARE_READ, NULL, OPEN_EXISTING,
 		FILE_ATTRIBUTE_NORMAL, NULL);
 	if (hFile == INVALID_HANDLE_VALUE) {
@@ -1486,8 +1403,8 @@ ApplicationType(
 
     if (applType == APPL_NONE) {
 	TclWinConvertError(GetLastError());
-	Tcl_AppendResult(interp, "couldn't execute \"", originalName,
-		"\": ", Tcl_PosixError(interp), (char *) NULL);
+	Tcl_SetObjResult(interp, Tcl_ObjPrintf("couldn't execute \"%s\": %s",
+		originalName, Tcl_PosixError(interp)));
 	return APPL_NONE;
     }
 
@@ -1499,9 +1416,8 @@ ApplicationType(
 	 * application name from the arguments.
 	 */
 
-	(*tclWinProcs->getShortPathNameProc)((TCHAR *) nativeFullPath,
-		nativeFullPath, MAX_PATH);
-	strcpy(fullName, Tcl_WinTCharToUtf((TCHAR *) nativeFullPath, -1, &ds));
+	GetShortPathName(nativeFullPath, nativeFullPath, MAX_PATH);
+	strcpy(fullName, Tcl_WinTCharToUtf(nativeFullPath, -1, &ds));
 	Tcl_DStringFree(&ds);
     }
     return applType;
@@ -1545,9 +1461,9 @@ BuildCommandLine(
      * Prime the path. Add a space separator if we were primed with something.
      */
 
-    Tcl_DStringAppend(&ds, Tcl_DStringValue(linePtr), -1);
+    TclDStringAppendDString(&ds, linePtr);
     if (Tcl_DStringLength(linePtr) > 0) {
-	Tcl_DStringAppend(&ds, " ", 1);
+	TclDStringAppendLiteral(&ds, " ");
     }
 
     for (i = 0; i < argc; i++) {
@@ -1555,7 +1471,7 @@ BuildCommandLine(
 	    arg = executable;
 	} else {
 	    arg = argv[i];
-	    Tcl_DStringAppend(&ds, " ", 1);
+	    TclDStringAppendLiteral(&ds, " ");
 	}
 
 	quote = 0;
@@ -1564,6 +1480,7 @@ BuildCommandLine(
 	} else {
 	    int count;
 	    Tcl_UniChar ch;
+
 	    for (start = arg; *start != '\0'; start += count) {
 		count = Tcl_UtfToUniChar(start, &ch);
 		if (Tcl_UniCharIsSpace(ch)) {	/* INTL: ISO space. */
@@ -1573,7 +1490,7 @@ BuildCommandLine(
 	    }
 	}
 	if (quote) {
-	    Tcl_DStringAppend(&ds, "\"", 1);
+	    TclDStringAppendLiteral(&ds, "\"");
 	}
 	start = arg;
 	for (special = arg; ; ) {
@@ -1602,7 +1519,7 @@ BuildCommandLine(
 	    }
 	    if (*special == '"') {
 		Tcl_DStringAppend(&ds, start, (int) (special - start));
-		Tcl_DStringAppend(&ds, "\\\"", 2);
+		TclDStringAppendLiteral(&ds, "\\\"");
 		start = special + 1;
 	    }
 	    if (*special == '\0') {
@@ -1612,7 +1529,7 @@ BuildCommandLine(
 	}
 	Tcl_DStringAppend(&ds, start, (int) (special - start));
 	if (quote) {
-	    Tcl_DStringAppend(&ds, "\"", 1);
+	    TclDStringAppendLiteral(&ds, "\"");
 	}
     }
     Tcl_DStringFree(linePtr);
@@ -1648,7 +1565,7 @@ TclpCreateCommandChannel(
 {
     char channelName[16 + TCL_INTEGER_SPACE];
     DWORD id;
-    PipeInfo *infoPtr = (PipeInfo *) ckalloc((unsigned) sizeof(PipeInfo));
+    PipeInfo *infoPtr = ckalloc(sizeof(PipeInfo));
 
     PipeInit();
 
@@ -1663,7 +1580,7 @@ TclpCreateCommandChannel(
     infoPtr->writeBuf = 0;
     infoPtr->writeBufLen = 0;
     infoPtr->writeError = 0;
-    infoPtr->channel = (Tcl_Channel) NULL;
+    infoPtr->channel = NULL;
 
     infoPtr->validMask = 0;
 
@@ -1705,9 +1622,9 @@ TclpCreateCommandChannel(
      * unique, in case channels share handles (stdin/stdout).
      */
 
-    sprintf(channelName, "file%" TCL_I_MODIFIER "x", (size_t)infoPtr);
+    sprintf(channelName, "file%" TCL_I_MODIFIER "x", (size_t) infoPtr);
     infoPtr->channel = Tcl_CreateChannel(&pipeChannelType, channelName,
-	    (ClientData) infoPtr, infoPtr->validMask);
+	    infoPtr, infoPtr->validMask);
 
     /*
      * Pipes have AUTO translation mode on Windows and ^Z eof char, which
@@ -1715,13 +1632,55 @@ TclpCreateCommandChannel(
      * Windows programs that expect a ^Z at EOF.
      */
 
-    Tcl_SetChannelOption((Tcl_Interp *) NULL, infoPtr->channel,
-	    "-translation", "auto");
-    Tcl_SetChannelOption((Tcl_Interp *) NULL, infoPtr->channel,
-	    "-eofchar", "\032 {}");
+    Tcl_SetChannelOption(NULL, infoPtr->channel, "-translation", "auto");
+    Tcl_SetChannelOption(NULL, infoPtr->channel, "-eofchar", "\032 {}");
     return infoPtr->channel;
 }
 
+/*
+ *----------------------------------------------------------------------
+ *
+ * Tcl_CreatePipe --
+ *
+ *	System dependent interface to create a pipe for the [chan pipe]
+ *	command. Stolen from TclX.
+ *
+ * Results:
+ *	TCL_OK or TCL_ERROR.
+ *
+ *----------------------------------------------------------------------
+ */
+
+int
+Tcl_CreatePipe(
+    Tcl_Interp *interp,		/* Errors returned in result.*/
+    Tcl_Channel *rchan,		/* Where to return the read side. */
+    Tcl_Channel *wchan,		/* Where to return the write side. */
+    int flags)			/* Reserved for future use. */
+{
+    HANDLE readHandle, writeHandle;
+    SECURITY_ATTRIBUTES sec;
+
+    sec.nLength = sizeof(SECURITY_ATTRIBUTES);
+    sec.lpSecurityDescriptor = NULL;
+    sec.bInheritHandle = FALSE;
+
+    if (!CreatePipe(&readHandle, &writeHandle, &sec, 0)) {
+	TclWinConvertError(GetLastError());
+	Tcl_SetObjResult(interp, Tcl_ObjPrintf(
+		"pipe creation failed: %s", Tcl_PosixError(interp)));
+	return TCL_ERROR;
+    }
+
+    *rchan = Tcl_MakeFileChannel((ClientData) readHandle, TCL_READABLE);
+    Tcl_RegisterChannel(interp, *rchan);
+
+    *wchan = Tcl_MakeFileChannel((ClientData) writeHandle, TCL_WRITABLE);
+    Tcl_RegisterChannel(interp, *wchan);
+
+    return TCL_OK;
+}
+
 /*
  *----------------------------------------------------------------------
  *
@@ -1746,8 +1705,8 @@ TclGetAndDetachPids(
 {
     PipeInfo *pipePtr;
     const Tcl_ChannelType *chanTypePtr;
+    Tcl_Obj *pidsObj;
     int i;
-    char buf[TCL_INTEGER_SPACE];
 
     /*
      * Punt if the channel is not a command channel.
@@ -1758,14 +1717,17 @@ TclGetAndDetachPids(
 	return;
     }
 
-    pipePtr = (PipeInfo *) Tcl_GetChannelInstanceData(chan);
+    pipePtr = Tcl_GetChannelInstanceData(chan);
+    TclNewObj(pidsObj);
     for (i = 0; i < pipePtr->numPids; i++) {
-	wsprintfA(buf, "%lu", TclpGetPid(pipePtr->pidPtr[i]));
-	Tcl_AppendElement(interp, buf);
-	Tcl_DetachPids(1, &(pipePtr->pidPtr[i]));
+	Tcl_ListObjAppendElement(NULL, pidsObj,
+		Tcl_NewWideIntObj((unsigned)
+			TclpGetPid(pipePtr->pidPtr[i])));
+	Tcl_DetachPids(1, &pipePtr->pidPtr[i]);
     }
+    Tcl_SetObjResult(interp, pidsObj);
     if (pipePtr->numPids > 0) {
-	ckfree((char *) pipePtr->pidPtr);
+	ckfree(pipePtr->pidPtr);
 	pipePtr->numPids = 0;
     }
 }
@@ -1910,12 +1872,26 @@ PipeClose2Proc(
 	    && (pipePtr->writeFile != NULL)) {
 	if (pipePtr->writeThread) {
 	    /*
-	     * Wait for the writer thread to finish the current buffer, then
-	     * terminate the thread and close the handles. If the channel is
-	     * nonblocking, there should be no pending write operations.
+	     * Wait for the  writer thread to finish the  current buffer, then
+	     * terminate the thread  and close the handles. If  the channel is
+	     * nonblocking but blocked during  exit, bail out since the worker
+	     * thread is not interruptible and we want TIP#398-fast-exit.
 	     */
+	    if (TclInExit()
+		&& (pipePtr->flags & PIPE_ASYNC)) {
 
-	    WaitForSingleObject(pipePtr->writable, INFINITE);
+		/* give it a chance to leave honorably */
+		SetEvent(pipePtr->stopWriter);
+
+		if (WaitForSingleObject(pipePtr->writable, 0) == WAIT_TIMEOUT) {
+		    return EAGAIN;
+		}
+
+	    } else {
+
+		WaitForSingleObject(pipePtr->writable, INFINITE);
+
+	    }
 
 	    /*
 	     * The thread may already have closed on it's own. Check its exit
@@ -2025,12 +2001,11 @@ PipeClose2Proc(
 	 */
 
 	if (pipePtr->errorFile) {
-	    WinFile *filePtr;
+	    WinFile *filePtr = (WinFile *) pipePtr->errorFile;
 
-	    filePtr = (WinFile*)pipePtr->errorFile;
 	    errChan = Tcl_MakeFileChannel((ClientData) filePtr->handle,
 		    TCL_READABLE);
-	    ckfree((char *) filePtr);
+	    ckfree(filePtr);
 	} else {
 	    errChan = NULL;
 	}
@@ -2040,14 +2015,14 @@ PipeClose2Proc(
     }
 
     if (pipePtr->numPids > 0) {
-	ckfree((char *) pipePtr->pidPtr);
+	ckfree(pipePtr->pidPtr);
     }
 
     if (pipePtr->writeBuf != NULL) {
 	ckfree(pipePtr->writeBuf);
     }
 
-    ckfree((char*) pipePtr);
+    ckfree(pipePtr);
 
     if (errorCode == 0) {
 	return result;
@@ -2215,7 +2190,7 @@ PipeOutputProc(
 		ckfree(infoPtr->writeBuf);
 	    }
 	    infoPtr->writeBufLen = toWrite;
-	    infoPtr->writeBuf = ckalloc((unsigned int) toWrite);
+	    infoPtr->writeBuf = ckalloc(toWrite);
 	}
 	memcpy(infoPtr->writeBuf, buf, (size_t) toWrite);
 	infoPtr->toWrite = toWrite;
@@ -2594,7 +2569,7 @@ Tcl_WaitPid(
      */
 
     CloseHandle(infoPtr->hProcess);
-    ckfree((char*)infoPtr);
+    ckfree(infoPtr);
 
     return result;
 }
@@ -2620,9 +2595,9 @@ Tcl_WaitPid(
 void
 TclWinAddProcess(
     void *hProcess,		/* Handle to process */
-    unsigned long id)	/* Global process identifier */
+    unsigned long id)		/* Global process identifier */
 {
-    ProcInfo *procPtr = (ProcInfo *) ckalloc(sizeof(ProcInfo));
+    ProcInfo *procPtr = ckalloc(sizeof(ProcInfo));
 
     PipeInit();
 
@@ -2664,15 +2639,13 @@ Tcl_PidObjCmd(
     PipeInfo *pipePtr;
     int i;
     Tcl_Obj *resultPtr;
-    char buf[TCL_INTEGER_SPACE];
 
     if (objc > 2) {
 	Tcl_WrongNumArgs(interp, 1, objv, "?channelId?");
 	return TCL_ERROR;
     }
     if (objc == 1) {
-	wsprintfA(buf, "%lu", (unsigned long) getpid());
-	Tcl_SetObjResult(interp, Tcl_NewStringObj(buf, -1));
+	Tcl_SetObjResult(interp, Tcl_NewWideIntObj((unsigned) getpid()));
     } else {
 	chan = Tcl_GetChannel(interp, Tcl_GetStringFromObj(objv[1], NULL),
 		NULL);
@@ -2687,9 +2660,9 @@ Tcl_PidObjCmd(
 	pipePtr = (PipeInfo *) Tcl_GetChannelInstanceData(chan);
 	resultPtr = Tcl_NewObj();
 	for (i = 0; i < pipePtr->numPids; i++) {
-	    wsprintfA(buf, "%lu", TclpGetPid(pipePtr->pidPtr[i]));
 	    Tcl_ListObjAppendElement(/*interp*/ NULL, resultPtr,
-		    Tcl_NewStringObj(buf, -1));
+		    Tcl_NewWideIntObj((unsigned)
+			    TclpGetPid(pipePtr->pidPtr[i])));
 	}
 	Tcl_SetObjResult(interp, resultPtr);
     }
@@ -2981,6 +2954,10 @@ PipeWriterThread(
 	     * an error, so exit.
 	     */
 
+	    if (waitResult == WAIT_OBJECT_0) {
+		SetEvent(infoPtr->writable);
+	    }
+
 	    break;
 	}
 
@@ -3080,6 +3057,100 @@ PipeThreadActionProc(
     Tcl_MutexUnlock(&pipeMutex);
 }
 
+/*
+ *----------------------------------------------------------------------
+ *
+ * TclpOpenTemporaryFile --
+ *
+ *	Creates a temporary file, possibly based on the supplied bits and
+ *	pieces of template supplied in the first three arguments. If the
+ *	fourth argument is non-NULL, it contains a Tcl_Obj to store the name
+ *	of the temporary file in (and it is caller's responsibility to clean
+ *	up). If the fourth argument is NULL, try to arrange for the temporary
+ *	file to go away once it is no longer needed.
+ *
+ * Results:
+ *	A read-write Tcl Channel open on the file.
+ *
+ *----------------------------------------------------------------------
+ */
+
+Tcl_Channel
+TclpOpenTemporaryFile(
+    Tcl_Obj *dirObj,
+    Tcl_Obj *basenameObj,
+    Tcl_Obj *extensionObj,
+    Tcl_Obj *resultingNameObj)
+{
+    TCHAR name[MAX_PATH];
+    char *namePtr;
+    HANDLE handle;
+    DWORD flags = FILE_ATTRIBUTE_TEMPORARY;
+    int length, counter, counter2;
+    Tcl_DString buf;
+
+    if (!resultingNameObj) {
+	flags |= FILE_FLAG_DELETE_ON_CLOSE;
+    }
+
+    namePtr = (char *) name;
+    length = GetTempPath(MAX_PATH, name);
+    if (length == 0) {
+	goto gotError;
+    }
+    namePtr += length * sizeof(TCHAR);
+    if (basenameObj) {
+	const char *string = Tcl_GetStringFromObj(basenameObj, &length);
+
+	Tcl_WinUtfToTChar(string, length, &buf);
+	memcpy(namePtr, Tcl_DStringValue(&buf), Tcl_DStringLength(&buf));
+	namePtr += Tcl_DStringLength(&buf);
+	Tcl_DStringFree(&buf);
+    } else {
+	const TCHAR *baseStr = TEXT("TCL");
+	int length = 3 * sizeof(TCHAR);
+
+	memcpy(namePtr, baseStr, length);
+	namePtr += length;
+    }
+    counter = TclpGetClicks() % 65533;
+    counter2 = 1024;			/* Only try this many times! Prevents
+					 * an infinite loop. */
+
+    do {
+	char number[TCL_INTEGER_SPACE + 4];
+
+	sprintf(number, "%d.TMP", counter);
+	counter = (unsigned short) (counter + 1);
+	Tcl_WinUtfToTChar(number, strlen(number), &buf);
+	Tcl_DStringSetLength(&buf, Tcl_DStringLength(&buf) + 1);
+	memcpy(namePtr, Tcl_DStringValue(&buf), Tcl_DStringLength(&buf) + 1);
+	Tcl_DStringFree(&buf);
+
+	handle = CreateFile(name,
+		GENERIC_READ|GENERIC_WRITE, 0, NULL, CREATE_NEW, flags, NULL);
+    } while (handle == INVALID_HANDLE_VALUE
+	    && --counter2 > 0
+	    && GetLastError() == ERROR_FILE_EXISTS);
+    if (handle == INVALID_HANDLE_VALUE) {
+	goto gotError;
+    }
+
+    if (resultingNameObj) {
+	Tcl_Obj *tmpObj = TclpNativeToNormalized(name);
+
+	Tcl_AppendObjToObj(resultingNameObj, tmpObj);
+	TclDecrRefCount(tmpObj);
+    }
+
+    return Tcl_MakeFileChannel((ClientData) handle,
+	    TCL_READABLE|TCL_WRITABLE);
+
+  gotError:
+    TclWinConvertError(GetLastError());
+    return NULL;
+}
+
 /*
  * Local Variables:
  * mode: c
