@@ -173,16 +173,16 @@ static ThreadSpecificData *SerialInit(void);
 static int		SerialInputProc(ClientData instanceData, char *buf,
 			    int toRead, int *errorCode);
 static int		SerialOutputProc(ClientData instanceData,
-			    CONST char *buf, int toWrite, int *errorCode);
+			    const char *buf, int toWrite, int *errorCode);
 static void		SerialSetupProc(ClientData clientData, int flags);
 static void		SerialWatchProc(ClientData instanceData, int mask);
 static void		ProcExitHandler(ClientData clientData);
 static int		SerialGetOptionProc(ClientData instanceData,
-			    Tcl_Interp *interp, CONST char *optionName,
+			    Tcl_Interp *interp, const char *optionName,
 			    Tcl_DString *dsPtr);
 static int		SerialSetOptionProc(ClientData instanceData,
-			    Tcl_Interp *interp, CONST char *optionName,
-			    CONST char *value);
+			    Tcl_Interp *interp, const char *optionName,
+			    const char *value);
 static DWORD WINAPI	SerialWriterThread(LPVOID arg);
 static void		SerialThreadActionProc(ClientData instanceData,
 			    int action);
@@ -197,7 +197,7 @@ static int		SerialBlockingWrite(SerialInfo *infoPtr, LPVOID buf,
  * based IO.
  */
 
-static Tcl_ChannelType serialChannelType = {
+static const Tcl_ChannelType serialChannelType = {
     "serial",			/* Type name. */
     TCL_CHANNEL_VERSION_5,	/* v5 channel */
     SerialCloseProc,		/* Close proc. */
@@ -214,7 +214,7 @@ static Tcl_ChannelType serialChannelType = {
     NULL,			/* handler proc. */
     NULL,			/* wide seek proc */
     SerialThreadActionProc,	/* thread action proc */
-    NULL,                       /* truncate */
+    NULL                       /* truncate */
 };
 
 /*
@@ -374,7 +374,7 @@ SerialGetMilliseconds(void)
 {
     Tcl_Time time;
 
-    TclpGetTime(&time);
+    Tcl_GetTime(&time);
 
     return (time.sec * 1000 + time.usec / 1000);
 }
@@ -527,7 +527,7 @@ SerialCheckProc(
 
 	if (needEvent) {
 	    infoPtr->flags |= SERIAL_PENDING;
-	    evPtr = (SerialEvent *) ckalloc(sizeof(SerialEvent));
+	    evPtr = ckalloc(sizeof(SerialEvent));
 	    evPtr->header.proc = SerialEventProc;
 	    evPtr->infoPtr = infoPtr;
 	    Tcl_QueueEvent((Tcl_Event *) evPtr, TCL_QUEUE_TAIL);
@@ -706,7 +706,7 @@ SerialCloseProc(
 	ckfree(serialPtr->writeBuf);
 	serialPtr->writeBuf = NULL;
     }
-    ckfree((char*) serialPtr);
+    ckfree(serialPtr);
 
     if (errorCode == 0) {
 	return result;
@@ -996,7 +996,7 @@ SerialInputProc(
 static int
 SerialOutputProc(
     ClientData instanceData,	/* Serial state. */
-    CONST char *buf,		/* The data buffer. */
+    const char *buf,		/* The data buffer. */
     int toWrite,		/* How many bytes to write? */
     int *errorCode)		/* Where to store error code. */
 {
@@ -1034,7 +1034,7 @@ SerialOutputProc(
 	 * the channel is in non-blocking mode.
 	 */
 
-	errno = EWOULDBLOCK;
+	errno = EAGAIN;
 	goto error1;
     }
 
@@ -1071,7 +1071,7 @@ SerialOutputProc(
 		ckfree(infoPtr->writeBuf);
 	    }
 	    infoPtr->writeBufLen = toWrite;
-	    infoPtr->writeBuf = ckalloc((unsigned int) toWrite);
+	    infoPtr->writeBuf = ckalloc(toWrite);
 	}
 	memcpy(infoPtr->writeBuf, buf, (size_t) toWrite);
 	infoPtr->toWrite = toWrite;
@@ -1410,45 +1410,40 @@ SerialWriterThread(
 /*
  *----------------------------------------------------------------------
  *
- * TclWinSerialOpen --
+ * TclWinSerialReopen --
  *
- *	Opens or Reopens the serial port with the OVERLAPPED FLAG set
+ *	Reopens the serial port with the OVERLAPPED FLAG set
  *
  * Results:
- *	Returns the new handle, or INVALID_HANDLE_VALUE. 
- *	If an existing channel is specified it is closed and reopened.
+ *	Returns the new handle, or INVALID_HANDLE_VALUE. Normally there
+ *	shouldn't be any error, because the same channel has previously been
+ *	succeesfully opened.
  *
  * Side effects:
- *	May close/reopen the original handle
+ *	May close the original handle
  *
  *----------------------------------------------------------------------
  */
 
 HANDLE
-TclWinSerialOpen(
+TclWinSerialReopen(
     HANDLE handle,
-    CONST TCHAR *name,
+    const TCHAR *name,
     DWORD access)
 {
     SerialInit();
 
-    /*
-     * If an open channel is specified, close it
-     */
-
-    if ( handle != INVALID_HANDLE_VALUE && CloseHandle(handle) == FALSE) {
-	return INVALID_HANDLE_VALUE;
-    }
-
     /*
      * Multithreaded I/O needs the overlapped flag set otherwise
      * ClearCommError blocks under Windows NT/2000 until serial output is
      * finished
      */
 
-    handle = (*tclWinProcs->createFileProc)(name, access, 0, 0,
-	    OPEN_EXISTING, FILE_FLAG_OVERLAPPED, 0);
-
+    if (CloseHandle(handle) == FALSE) {
+	return INVALID_HANDLE_VALUE;
+    }
+    handle = CreateFile(name, access, 0, 0, OPEN_EXISTING,
+	    FILE_FLAG_OVERLAPPED, 0);
     return handle;
 }
 
@@ -1481,7 +1476,7 @@ TclWinOpenSerialChannel(
 
     SerialInit();
 
-    infoPtr = (SerialInfo *) ckalloc((unsigned) sizeof(SerialInfo));
+    infoPtr = ckalloc(sizeof(SerialInfo));
     memset(infoPtr, 0, sizeof(SerialInfo));
 
     infoPtr->validMask = permissions;
@@ -1502,10 +1497,10 @@ TclWinOpenSerialChannel(
      * are shared between multiple channels (stdin/stdout).
      */
 
-    sprintf(channelName, "file%" TCL_I_MODIFIER "x", (size_t)infoPtr);
+    sprintf(channelName, "file%" TCL_I_MODIFIER "x", (size_t) infoPtr);
 
     infoPtr->channel = Tcl_CreateChannel(&serialChannelType, channelName,
-	    (ClientData) infoPtr, permissions);
+	    infoPtr, permissions);
 
 
     SetupComm(handle, infoPtr->sysBufRead, infoPtr->sysBufWrite);
@@ -1648,17 +1643,17 @@ static int
 SerialSetOptionProc(
     ClientData instanceData,	/* File state. */
     Tcl_Interp *interp,		/* For error reporting - can be NULL. */
-    CONST char *optionName,	/* Which option to set? */
-    CONST char *value)		/* New value for option. */
+    const char *optionName,	/* Which option to set? */
+    const char *value)		/* New value for option. */
 {
     SerialInfo *infoPtr;
     DCB dcb;
     BOOL result, flag;
     size_t len, vlen;
     Tcl_DString ds;
-    CONST TCHAR *native;
+    const TCHAR *native;
     int argc;
-    CONST char **argv;
+    const char **argv;
 
     infoPtr = (SerialInfo *) instanceData;
 
@@ -1676,19 +1671,18 @@ SerialSetOptionProc(
 
     if ((len > 2) && (strncmp(optionName, "-mode", len) == 0)) {
 	if (!GetCommState(infoPtr->handle, &dcb)) {
-	    if (interp != NULL) {
-		Tcl_AppendResult(interp, "can't get comm state", NULL);
-	    }
-	    return TCL_ERROR;
+	    goto getStateFailed;
 	}
 	native = Tcl_WinUtfToTChar(value, -1, &ds);
-	result = (*tclWinProcs->buildCommDCBProc)(native, &dcb);
+	result = BuildCommDCB(native, &dcb);
 	Tcl_DStringFree(&ds);
 
 	if (result == FALSE) {
 	    if (interp != NULL) {
-		Tcl_AppendResult(interp, "bad value \"", value,
-			"\" for -mode: should be baud,parity,data,stop", NULL);
+		Tcl_SetObjResult(interp, Tcl_ObjPrintf(
+			"bad value \"%s\" for -mode: should be baud,parity,data,stop",
+			value));
+		Tcl_SetErrorCode(interp, "TCL", "VALUE", "SERIALMODE", NULL);
 	    }
 	    return TCL_ERROR;
 	}
@@ -1703,10 +1697,7 @@ SerialSetOptionProc(
 	dcb.fAbortOnError = FALSE;
 
 	if (!SetCommState(infoPtr->handle, &dcb)) {
-	    if (interp != NULL) {
-		Tcl_AppendResult(interp, "can't set comm state", NULL);
-	    }
-	    return TCL_ERROR;
+	    goto setStateFailed;
 	}
 	return TCL_OK;
     }
@@ -1717,10 +1708,7 @@ SerialSetOptionProc(
 
     if ((len > 1) && (strncmp(optionName, "-handshake", len) == 0)) {
 	if (!GetCommState(infoPtr->handle, &dcb)) {
-	    if (interp != NULL) {
-		Tcl_AppendResult(interp, "can't get comm state", NULL);
-	    }
-	    return TCL_ERROR;
+	    goto getStateFailed;
 	}
 
 	/*
@@ -1755,18 +1743,16 @@ SerialSetOptionProc(
 	    dcb.fDtrControl = DTR_CONTROL_HANDSHAKE;
 	} else {
 	    if (interp != NULL) {
-		Tcl_AppendResult(interp, "bad value \"", value,
-			"\" for -handshake: must be one of xonxoff, rtscts, "
-			"dtrdsr or none", NULL);
+		Tcl_SetObjResult(interp, Tcl_ObjPrintf(
+			"bad value \"%s\" for -handshake: must be one of"
+			" xonxoff, rtscts, dtrdsr or none", value));
+		Tcl_SetErrorCode(interp, "TCL", "VALUE", "HANDSHAKE", NULL);
 	    }
 	    return TCL_ERROR;
 	}
 
 	if (!SetCommState(infoPtr->handle, &dcb)) {
-	    if (interp != NULL) {
-		Tcl_AppendResult(interp, "can't set comm state", NULL);
-	    }
-	    return TCL_ERROR;
+	    goto setStateFailed;
 	}
 	return TCL_OK;
     }
@@ -1777,10 +1763,7 @@ SerialSetOptionProc(
 
     if ((len > 1) && (strncmp(optionName, "-xchar", len) == 0)) {
 	if (!GetCommState(infoPtr->handle, &dcb)) {
-	    if (interp != NULL) {
-		Tcl_AppendResult(interp, "can't get comm state", NULL);
-	    }
-	    return TCL_ERROR;
+	    goto getStateFailed;
 	}
 
 	if (Tcl_SplitList(interp, value, &argc, &argv) == TCL_ERROR) {
@@ -1789,11 +1772,12 @@ SerialSetOptionProc(
 	if (argc != 2) {
 	badXchar:
 	    if (interp != NULL) {
-		Tcl_AppendResult(interp, "bad value for -xchar: should be "
-			"a list of two elements with each a single character",
-			NULL);
+		Tcl_SetObjResult(interp, Tcl_NewStringObj(
+			"bad value for -xchar: should be a list of"
+			" two elements with each a single character", -1));
+		Tcl_SetErrorCode(interp, "TCL", "VALUE", "XCHAR", NULL);
 	    }
-	    ckfree((char *) argv);
+	    ckfree(argv);
 	    return TCL_ERROR;
 	}
 
@@ -1824,13 +1808,10 @@ SerialSetOptionProc(
 	    }
 	    dcb.XoffChar = (char) character;
 	}
-	ckfree((char *) argv);
+	ckfree(argv);
 
 	if (!SetCommState(infoPtr->handle, &dcb)) {
-	    if (interp != NULL) {
-		Tcl_AppendResult(interp, "can't set comm state", NULL);
-	    }
-	    return TCL_ERROR;
+	    goto setStateFailed;
 	}
 	return TCL_OK;
     }
@@ -1847,11 +1828,12 @@ SerialSetOptionProc(
 	}
 	if ((argc % 2) == 1) {
 	    if (interp != NULL) {
-		Tcl_AppendResult(interp, "bad value \"", value,
-			"\" for -ttycontrol: should be a list of "
-			"signal,value pairs", NULL);
+		Tcl_SetObjResult(interp, Tcl_ObjPrintf(
+			"bad value \"%s\" for -ttycontrol: should be "
+			"a list of signal,value pairs", value));
+		Tcl_SetErrorCode(interp, "TCL", "VALUE", "TTYCONTROL", NULL);
 	    }
-	    ckfree((char *) argv);
+	    ckfree(argv);
 	    return TCL_ERROR;
 	}
 
@@ -1864,7 +1846,10 @@ SerialSetOptionProc(
 		if (!EscapeCommFunction(infoPtr->handle,
 			(DWORD) (flag ? SETDTR : CLRDTR))) {
 		    if (interp != NULL) {
-			Tcl_AppendResult(interp, "can't set DTR signal", NULL);
+			Tcl_SetObjResult(interp, Tcl_NewStringObj(
+				"can't set DTR signal", -1));
+			Tcl_SetErrorCode(interp, "TCL", "OPERATION",
+				"FCONFIGURE", "TTY_SIGNAL", NULL);
 		    }
 		    result = TCL_ERROR;
 		    break;
@@ -1873,7 +1858,10 @@ SerialSetOptionProc(
 		if (!EscapeCommFunction(infoPtr->handle,
 			(DWORD) (flag ? SETRTS : CLRRTS))) {
 		    if (interp != NULL) {
-			Tcl_AppendResult(interp, "can't set RTS signal", NULL);
+			Tcl_SetObjResult(interp, Tcl_NewStringObj(
+				"can't set RTS signal", -1));
+			Tcl_SetErrorCode(interp, "TCL", "OPERATION",
+				"FCONFIGURE", "TTY_SIGNAL", NULL);
 		    }
 		    result = TCL_ERROR;
 		    break;
@@ -1882,15 +1870,20 @@ SerialSetOptionProc(
 		if (!EscapeCommFunction(infoPtr->handle,
 			(DWORD) (flag ? SETBREAK : CLRBREAK))) {
 		    if (interp != NULL) {
-			Tcl_AppendResult(interp,"can't set BREAK signal",NULL);
+			Tcl_SetObjResult(interp, Tcl_NewStringObj(
+				"can't set BREAK signal", -1));
+			Tcl_SetErrorCode(interp, "TCL", "OPERATION",
+				"FCONFIGURE", "TTY_SIGNAL", NULL);
 		    }
 		    result = TCL_ERROR;
 		    break;
 		}
 	    } else {
 		if (interp != NULL) {
-		    Tcl_AppendResult(interp, "bad signal name \"", argv[i],
-			    "\" for -ttycontrol: must be DTR, RTS or BREAK",
+		    Tcl_SetObjResult(interp, Tcl_ObjPrintf(
+			    "bad signal name \"%s\" for -ttycontrol: must be"
+			    " DTR, RTS or BREAK", argv[i]));
+		    Tcl_SetErrorCode(interp, "TCL", "VALUE", "TTY_SIGNAL",
 			    NULL);
 		}
 		result = TCL_ERROR;
@@ -1898,7 +1891,7 @@ SerialSetOptionProc(
 	    }
 	}
 
-	ckfree((char *) argv);
+	ckfree(argv);
 	return result;
     }
 
@@ -1924,20 +1917,24 @@ SerialSetOptionProc(
 	    inSize  = atoi(argv[0]);
 	    outSize = atoi(argv[1]);
 	}
-	ckfree((char *) argv);
+	ckfree(argv);
 
 	if ((argc < 1) || (argc > 2) || (inSize <= 0) || (outSize <= 0)) {
 	    if (interp != NULL) {
-		Tcl_AppendResult(interp, "bad value \"", value,
-			"\" for -sysbuffer: should be a list of one or two "
-			"integers > 0", NULL);
+		Tcl_SetObjResult(interp, Tcl_ObjPrintf(
+			"bad value \"%s\" for -sysbuffer: should be "
+			"a list of one or two integers > 0", value));
+		Tcl_SetErrorCode(interp, "TCL", "VALUE", "SYS_BUFFER", NULL);
 	    }
 	    return TCL_ERROR;
 	}
 
 	if (!SetupComm(infoPtr->handle, inSize, outSize)) {
 	    if (interp != NULL) {
-		Tcl_AppendResult(interp, "can't setup comm buffers", NULL);
+		TclWinConvertError(GetLastError());
+		Tcl_SetObjResult(interp, Tcl_ObjPrintf(
+			"can't setup comm buffers: %s",
+			Tcl_PosixError(interp)));
 	    }
 	    return TCL_ERROR;
 	}
@@ -1950,18 +1947,12 @@ SerialSetOptionProc(
 	 */
 
 	if (!GetCommState(infoPtr->handle, &dcb)) {
-	    if (interp != NULL) {
-		Tcl_AppendResult(interp, "can't get comm state", NULL);
-	    }
-	    return TCL_ERROR;
+	    goto getStateFailed;
 	}
 	dcb.XonLim = (WORD) (infoPtr->sysBufRead*1/2);
 	dcb.XoffLim = (WORD) (infoPtr->sysBufRead*1/4);
 	if (!SetCommState(infoPtr->handle, &dcb)) {
-	    if (interp != NULL) {
-		Tcl_AppendResult(interp, "can't set comm state", NULL);
-	    }
-	    return TCL_ERROR;
+	    goto setStateFailed;
 	}
 	return TCL_OK;
     }
@@ -1991,7 +1982,10 @@ SerialSetOptionProc(
 	tout.ReadTotalTimeoutConstant = msec;
 	if (!SetCommTimeouts(infoPtr->handle, &tout)) {
 	    if (interp != NULL) {
-		Tcl_AppendResult(interp, "can't set comm timeouts", NULL);
+		TclWinConvertError(GetLastError());
+		Tcl_SetObjResult(interp, Tcl_ObjPrintf(
+			"can't set comm timeouts: %s",
+			Tcl_PosixError(interp)));
 	    }
 	    return TCL_ERROR;
 	}
@@ -2001,6 +1995,22 @@ SerialSetOptionProc(
 
     return Tcl_BadChannelOption(interp, optionName,
 	    "mode handshake pollinterval sysbuffer timeout ttycontrol xchar");
+
+  getStateFailed:
+    if (interp != NULL) {
+	TclWinConvertError(GetLastError());
+	Tcl_SetObjResult(interp, Tcl_ObjPrintf(
+		"can't get comm state: %s", Tcl_PosixError(interp)));
+    }
+    return TCL_ERROR;
+
+  setStateFailed:
+    if (interp != NULL) {
+	TclWinConvertError(GetLastError());
+	Tcl_SetObjResult(interp, Tcl_ObjPrintf(
+		"can't set comm state: %s", Tcl_PosixError(interp)));
+    }
+    return TCL_ERROR;
 }
 
 /*
@@ -2028,7 +2038,7 @@ static int
 SerialGetOptionProc(
     ClientData instanceData,	/* File state. */
     Tcl_Interp *interp,		/* For error reporting - can be NULL. */
-    CONST char *optionName,	/* Option to get. */
+    const char *optionName,	/* Option to get. */
     Tcl_DString *dsPtr)		/* Where to store value(s). */
 {
     SerialInfo *infoPtr;
@@ -2053,12 +2063,14 @@ SerialGetOptionProc(
     }
     if (len==0 || (len>2 && (strncmp(optionName, "-mode", len) == 0))) {
 	char parity;
-	char *stop;
+	const char *stop;
 	char buf[2 * TCL_INTEGER_SPACE + 16];
 
 	if (!GetCommState(infoPtr->handle, &dcb)) {
 	    if (interp != NULL) {
-		Tcl_AppendResult(interp, "can't get comm state", NULL);
+		TclWinConvertError(GetLastError());
+		Tcl_SetObjResult(interp, Tcl_ObjPrintf(
+			"can't get comm state: %s", Tcl_PosixError(interp)));
 	    }
 	    return TCL_ERROR;
 	}
@@ -2126,7 +2138,9 @@ SerialGetOptionProc(
 
 	if (!GetCommState(infoPtr->handle, &dcb)) {
 	    if (interp != NULL) {
-		Tcl_AppendResult(interp, "can't get comm state", NULL);
+		TclWinConvertError(GetLastError());
+		Tcl_SetObjResult(interp, Tcl_ObjPrintf(
+			"can't get comm state: %s", Tcl_PosixError(interp)));
 	    }
 	    return TCL_ERROR;
 	}
@@ -2202,7 +2216,9 @@ SerialGetOptionProc(
 
 	if (!GetCommModemStatus(infoPtr->handle, &status)) {
 	    if (interp != NULL) {
-		Tcl_AppendResult(interp, "can't get tty status", NULL);
+		TclWinConvertError(GetLastError());
+		Tcl_SetObjResult(interp, Tcl_ObjPrintf(
+			"can't get tty status: %s", Tcl_PosixError(interp)));
 	    }
 	    return TCL_ERROR;
 	}
@@ -2212,10 +2228,9 @@ SerialGetOptionProc(
 
     if (valid) {
 	return TCL_OK;
-    } else {
-	return Tcl_BadChannelOption(interp, optionName,
-		"mode pollinterval lasterror queue sysbuffer ttystatus xchar");
     }
+    return Tcl_BadChannelOption(interp, optionName,
+	    "mode pollinterval lasterror queue sysbuffer ttystatus xchar");
 }
 
 /*
